/***************************************************************************
                          tea_text_document.c  -  description
                             -------------------
    begin                : Mon Dec 1 2003
    copyright            : (C) 2003, 2004 by Peter 'Roxton' Semiletov
    email                : tea@list.ru
 **************************************************************************/


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 **************************************-************************************/


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>

#include <errno.h>
#include <sys/stat.h>
#include <glib.h>
#include <time.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
                 
#include "tea_text_document.h"
#include "tea_funx.h"
#include "callbacks.h"
#include "interface.h"
#include "tea_config.h"
#include "tea_defs.h"
#include "tea_hl.h"
#include "tea_fr.h"
#include "tea_about.h"
#include "undo.h"


//n.p. Depeche Mode - Enjoy The Silence 

void set_fam_text (gchar *def)
{
  if (strlen (gtk_entry_get_text (ent_search)) == 0)
     {
      gtk_entry_set_text (ent_search, def);  
      log_to_memo (_("Read the manual!"), NULL, LM_ERROR);
     } 

  gl_famous = add_to_list_with_limit (gl_famous, gtk_entry_get_text (ent_search), confile.famous_history_max);
  if (gl_famous)
     gtk_combo_set_popdown_strings (GTK_COMBO (cmb_famous), gl_famous);
}


void on_mni_file_open_at_cursor (GtkMenuItem *menuitem,
                                 gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *w = get_c_url (cur_text_doc);
  handle_file (w, 0);
  g_free (w);
}


gboolean tea_init (gpointer data)
{
  gint i; 
  gchar *fn;

  for (i = 1; i <= g_argc; ++i)
      {
       fn = g_malloc (2048); 

       if (realpath (g_argv[i], fn))
          open_file_std (fn); 

       g_free (fn);
      }
}  


void on_mni_new_file_activate (GtkMenuItem *menuitem,
                               gpointer user_data)
{
  cur_text_doc = doc_clear_new ();
  
  gtk_window_set_title (GTK_WINDOW (tea_main_window), cur_text_doc->file_name);
  gtk_widget_grab_focus (cur_text_doc->text_view);
  tabs_reload ();
}


static void on_mni_dialog_control_box_bookmarks_goto (GtkMenuItem *menuitem,
                                                      gpointer user_data)
{
   gchar *f = gtk_widget_get_name (menuitem);
   gchar *d = g_strconcat (f, G_DIR_SEPARATOR_S, NULL);
   gtk_file_selection_set_filename ((GtkFileSelection*) user_data, d);
   g_free (d);
}


static GtkWidget* create_dialog_control_box_bookmarks_menu (gpointer parent)
{
  GtkWidget *mni_x = gtk_option_menu_new ();
  GtkWidget *mni_m = gtk_menu_new ();
  gtk_widget_show (mni_x);
  gtk_option_menu_set_menu (mni_x, mni_m);
   
  build_menu_wudata_from_glist (gl_tea_kwas_bookmarks, mni_m, on_mni_dialog_control_box_bookmarks_goto, parent);
  gtk_option_menu_set_history (mni_x, 0);

  return mni_x;
}


static void on_dialog_control_box_add_bookmark (GtkButton *button,
                                                gpointer user_data)
{
  gchar *s = gtk_file_selection_get_filename ((GtkFileSelection*) user_data);
  gchar *d = g_path_get_dirname (s);
  gl_tea_kwas_bookmarks = g_list_prepend (gl_tea_kwas_bookmarks, d);
}


static GtkWidget* create_dialog_control_box (t_ppair *p, gint mode)
{
  GtkWidget *label; 
  GtkWidget *vb = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_VBOX (p->b), vb, FALSE, FALSE, 0);
  gtk_widget_show (vb);


  GtkWidget *hb1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hb1);

  GtkWidget *hb2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hb2);

  GtkWidget *bookmarks = create_dialog_control_box_bookmarks_menu (p->a);

  GtkWidget *bt_add_bookmark = gtk_button_new_with_label ("+");
  gtk_widget_show (bt_add_bookmark);

  g_signal_connect ((gpointer) bt_add_bookmark, "clicked",
                    G_CALLBACK (on_dialog_control_box_add_bookmark),
                    p->a);


  gtk_box_pack_start (GTK_BOX (vb), hb1, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vb), hb2, FALSE, FALSE, 0);

  label = gtk_label_new (_("Bookmarks"));
  gtk_widget_show (label);

  gtk_box_pack_start (GTK_BOX (hb1), label, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hb1), bookmarks, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hb1), bt_add_bookmark, FALSE, FALSE, 0);

  GtkWidget *mni_enc = gtk_option_menu_new ();
  GtkWidget *mni_m = gtk_menu_new ();

  gtk_widget_show (mni_enc);
  gtk_option_menu_set_menu (mni_enc, mni_m);

  if (mode == 0 )
     {
      if (confile.do_enc_autodetect)
         cur_settings.selected_enc = ch_str (cur_settings.selected_enc, "autodetect"); 
      else
          cur_settings.selected_enc = ch_str (cur_settings.selected_enc, "UTF-8"); 

      build_menu_from_glist (confile.iconv_encs, mni_m, on_mni_new_file_open_select_enc);
     } 


  if (mode == 1)
     {
      cur_settings.selected_enc = ch_str (cur_settings.selected_enc, "UTF-8"); 
 
      GList *l = g_list_first (confile.iconv_encs);

      while (l)
           {
            if (strcmp ((gchar*) l->data, "autodetect") != 0)
                mni_temp = new_menu_item ((gchar*) l->data, mni_m, on_mni_new_file_open_select_enc);
            l = g_list_next (l);
           }

      //gtk_box_pack_start (bh2, mni_enc, FALSE, FALSE, 0);
      //gtk_option_menu_set_history (mni_enc, 0);
  

      //build_menu_from_glist (confile.iconv_encs, mni_m, on_mni_new_file_open_select_enc);
     } 

  label = gtk_label_new (_("Encoding: "));
  gtk_widget_show (label);
 

  gtk_box_pack_start (GTK_BOX (hb2), label, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hb2), mni_enc, FALSE, FALSE, 0);

  gtk_option_menu_set_history (mni_enc, 0);

  return vb;
}


void file_open_callback_widget (GtkWidget *widget,
                                gpointer user_data)
{
  gchar **files = gtk_file_selection_get_selections (user_data);

  if (files)
     {
      gint c = -1;
      while (files [++c])
            if (! g_file_test (files[c], G_FILE_TEST_IS_DIR))
              cur_text_doc = open_file_std (files[c]);
     }

  g_strfreev (files);
}


void on_mni_new_file_open_select_enc (GtkMenuItem *menuitem,
                                      gpointer user_data)
{
  cur_settings.selected_enc = ch_str (cur_settings.selected_enc, gtk_widget_get_name (menuitem));
}


//n.p. The Rolling Stones - Mother's Little Helper
void on_mni_file_open_activate (GtkMenuItem *menuitem,
                                gpointer user_data)
{
  gchar *dir;
  gchar *t;

  GtkFileSelection *file_selector = gtk_file_selection_new (_("File open"));
  gtk_file_selection_set_select_multiple (file_selector, TRUE);
 
  g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                    "clicked",
                    G_CALLBACK (file_open_callback_widget),
                    (gpointer) file_selector);
  
  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->cancel_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  /*GtkWidget *mni_enc = gtk_option_menu_new ();
  GtkWidget *mni_m = gtk_menu_new ();

  gtk_widget_show (mni_enc);
  gtk_option_menu_set_menu (mni_enc, mni_m);

  if (confile.do_enc_autodetect)
     cur_settings.selected_enc = ch_str (cur_settings.selected_enc, "autodetect"); 
  else
      cur_settings.selected_enc = ch_str (cur_settings.selected_enc, "UTF-8"); 

  build_menu_from_glist (confile.iconv_encs, mni_m, on_mni_new_file_open_select_enc);

  gtk_box_pack_start (file_selector->main_vbox, mni_enc, FALSE, FALSE, 0);
  gtk_option_menu_set_history (mni_enc, 0);
*/

  t_ppair *p = g_malloc (sizeof (t_ppair));
  p->a = file_selector;
  p->b = file_selector->main_vbox;

  GtkWidget *w = create_dialog_control_box (p, 0);

  g_free (p);
//  GtkWidget *w = create_dialog_control_box (file_selector->main_vbox);
  //gtk_box_pack_start (file_selector->main_vbox, w, FALSE, FALSE, 0);

  gtk_window_resize (file_selector, get_value (confile.screen_w, confile.saveopen_w), get_value (confile.screen_h, confile.saveopen_h));
  
  if (confile.use_def_open_dir)
      gtk_file_selection_set_filename (file_selector, confile.def_open_dir);
  else
      if (get_page_text())
         if (g_file_test (cur_text_doc->file_name, G_FILE_TEST_EXISTS))
            {
             t = g_path_get_dirname (cur_text_doc->file_name);
             dir = g_strconcat (t, G_DIR_SEPARATOR_S, NULL);
             gtk_file_selection_set_filename (file_selector, dir);
             g_free (dir);
             g_free (t);
            }

  gtk_widget_show (file_selector);
}


void on_mni_file_save_activate (GtkMenuItem *menuitem,
                                gpointer user_data)
{
  if (! get_page_text()) return;
  
  if (! cur_text_doc->b_saved)
     on_mni_file_save_as_activate (menuitem, user_data);
  else
      if (text_doc_save (cur_text_doc, cur_text_doc->file_name))
         log_to_memo (_("%s saved OK"), cur_text_doc->file_name, LM_NORMAL);
}


void file_save_as_callback_widget (GtkWidget *widget,
                                   gpointer user_data)
{
  if (! get_page_text()) return;

  cur_text_doc->file_name = ch_str (cur_text_doc->file_name, gtk_file_selection_get_filename (GTK_FILE_SELECTION (user_data)));
  cur_text_doc->encoding = ch_str (cur_text_doc->encoding, cur_settings.selected_enc);  

  if (! text_doc_save (cur_text_doc, cur_text_doc->file_name))
     {
      log_to_memo (_("error by saving %s!!!"), cur_text_doc->file_name, LM_ERROR);
      return;
     }
      
  cur_text_doc->b_saved = TRUE;

  gtk_label_set_label (cur_text_doc->tab_label, basename (cur_text_doc->file_name));
  set_title (cur_text_doc);

  if (strstr (cur_text_doc->file_name, confile.snippets_dir))
     reload_snippets();

  if (strstr (cur_text_doc->file_name, confile.templates_dir))
     reload_templates();

  tabs_reload ();
}


void on_mni_file_save_as_activate (GtkMenuItem *menuitem,
                                   gpointer user_data)
{
  if (get_page_text ())
     show_save_as_dlg (0);
}


void on_mni_out_activate (GtkMenuItem *menuitem,
                          gpointer user_data)
{
  ui_done();
  gtk_main_quit ();
}


void on_mni_case_upcase_activate (GtkMenuItem *menuitem,
                                  gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);
  gchar *t;

  if (buf)
     {
      t = g_utf8_strup (buf, -1);
      doc_rep_sel (cur_text_doc, t);
      g_free (buf);
      g_free (t);
     }
  else
      { 
       GtkTextIter itstart;
       GtkTextIter itend; 
       buf = doc_get_current_word (cur_text_doc, &itstart, &itend);

       if (! buf)
          return;
  
       t = g_utf8_strup (buf, -1);
      
       doc_set_new_text (cur_text_doc, &itstart, &itend, t);        
                                                    
       g_free (buf);
       g_free (t);
      }
}


void on_mni_case_locase_activate (GtkMenuItem *menuitem,
                                  gpointer user_data) 
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);
  gchar *t;

  if (buf)
     {
      t = g_utf8_strdown (buf, -1);
      doc_rep_sel (cur_text_doc, t);
      g_free (buf);
      g_free (t);
     }
  else
      {
       GtkTextIter itstart;
       GtkTextIter itend; 
       buf = doc_get_current_word (cur_text_doc, &itstart, &itend);

       if (! buf)
          return;

       t = g_utf8_strdown (buf, -1);
      
       doc_set_new_text (cur_text_doc, &itstart, &itend, t);        
                                                    
       g_free (buf);
       g_free (t);
      }
}


void on_mni_reverse (GtkMenuItem *menuitem,
                     gpointer user_data) 
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);
  if (! buf) return;

  gchar *t = g_utf8_strreverse (buf, -1);
  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
}


void on_mni_file_save_version (GtkMenuItem *menuitem,
                               gpointer user_data) 
{
  if (! get_page_text()) return;
  
  gchar *s = get_file_ver_name (cur_text_doc->file_name);
  if (! s)
     {
      log_to_memo(_("Save the file first..."), NULL, LM_ADVICE);
      return;
     }

  text_doc_save (cur_text_doc, s);
  log_to_memo (_("%s saved good, don't worry..."), s, LM_NORMAL);
  g_free (s);
}


void on_mni_Markup_bold_activate (GtkMenuItem *menuitem,
                                  gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *t = NULL;
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf)
      {
       doc_insert_at_cursor (cur_text_doc, "<b></b>");
       doc_move_cursor_backw_middle_tags (cur_text_doc);
      }
  else
      {
       t = g_strconcat ("<b>", buf, "</b>", NULL);
       doc_rep_sel (cur_text_doc, t);
      }

  g_free (buf);
  g_free (t);
}


void on_mni_Markup_link_activate (GtkMenuItem *menuitem,
                                  gpointer user_data) 
{
  if (! get_page_text()) return;
  
  gchar *t = NULL;
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf)
     {
      doc_insert_at_cursor (cur_text_doc, "<a href=\"\"></a>");
      doc_move_cursor_backw_middle_tags (cur_text_doc);
     }
  else
      {
       t = g_strconcat ("<a href=\"\">", buf, "</a>", NULL);
       doc_rep_sel (cur_text_doc, t);
       doc_move_to_pos_bw_quote (cur_text_doc);
      }

  g_free (buf);
  g_free (t);
}


void on_mni_Markup_italic_activate (GtkMenuItem *menuitem,
                                    gpointer user_data) 
{
  if (! get_page_text()) return;
  
  gchar *t = NULL;
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf)
     {
      doc_insert_at_cursor (cur_text_doc, "<i></i>");
      doc_move_cursor_backw_middle_tags (cur_text_doc);
     }
  else
      {
       t = g_strconcat ("<i>", buf, "</i>", NULL);
       doc_rep_sel (cur_text_doc, t);
       g_free (t);
      }

  g_free (buf);
}


void on_mni_Markup_underline_activate (GtkMenuItem *menuitem,
                                       gpointer user_data) 
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);
  if (! buf)
      {
       doc_insert_at_cursor (cur_text_doc, "<u></u>");
       doc_move_cursor_backw_middle_tags (cur_text_doc);
      }

  gchar *t = g_strconcat ("<u>", buf, "</u>", NULL);
  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
}


void on_mni_Markup_para_activate (GtkMenuItem *menuitem,
                                  gpointer user_data) 
{
  if (! get_page_text()) return;
  
  gchar *t = NULL;
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf)
      {
       doc_insert_at_cursor (cur_text_doc, "<p></p>");
       doc_move_cursor_backw_middle_tags (cur_text_doc);
      }
  else
      {
       t = g_strconcat ("<p>", buf, "</p>", NULL);
       doc_rep_sel (cur_text_doc, t);
       g_free (t);
      }

  g_free (buf);
}


void on_mni_Markup_header_activate (GtkMenuItem *menuitem,
                                    gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *t = NULL;
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf)
     {
      t = g_strconcat ("<", gtk_widget_get_name (menuitem), ">",
                       "</", gtk_widget_get_name (menuitem), ">",
                       NULL);
      doc_insert_at_cursor (cur_text_doc, t);
      doc_move_cursor_backw_middle_tags (cur_text_doc);
     }
  else
      {
       t = g_strconcat ("<", gtk_widget_get_name (menuitem), ">",
                        buf,
                        "</", gtk_widget_get_name (menuitem), ">",
                        NULL);
       doc_rep_sel (cur_text_doc, t);
      }

  g_free (buf);
  g_free (t);
}


static gint last_page = -1;

void on_notebook1_switch_page (GtkNotebook *notebook,
                               GtkNotebookPage *page,
                               guint page_num,
                               gpointer user_data)
{
  t_note_page *dc = get_page_by_index (page_num);
  
  if (dc)
     {
      set_title (dc);
      gtk_widget_grab_focus (dc->text_view);
      last_page = gtk_notebook_get_current_page (notebook);
     }
}


void on_mni_file_close_current (GtkMenuItem *menuitem,
                                gpointer user_data)
{
  page_del_by_index (gtk_notebook_get_current_page ((GtkNotebook *) notebook1));
}


void on_mni_view_wrap (GtkMenuItem *menuitem,
                       gpointer user_data)
{
  if (! get_page_text()) return;
  
  if (gtk_text_view_get_wrap_mode (cur_text_doc->text_view) != GTK_WRAP_WORD)
      gtk_text_view_set_wrap_mode (cur_text_doc->text_view, GTK_WRAP_WORD);
  else
      gtk_text_view_set_wrap_mode (cur_text_doc->text_view, GTK_WRAP_NONE);
}


void on_mni_Markup_br (GtkMenuItem *menuitem,
                       gpointer user_data)
{
  if (! get_page_text()) return;
  if (! confile.xhtml_mode)
     doc_insert_at_cursor (cur_text_doc, "<br>");
  else
      doc_insert_at_cursor (cur_text_doc, "<br />"); 
}
                       

void on_mni_Functions_number_arabian2roman (GtkMenuItem *menuitem,
                                            gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);
  if (! buf) return;

  gchar *t = g_malloc (1024);

  tNumber2Roman (strtol (buf, NULL, 10), TRUE, t);

  if (t)
     doc_rep_sel (cur_text_doc, t);

  g_free (t);
  g_free (buf);
}


void on_mni_view_show_line_numbers (GtkMenuItem *menuitem,
                                    gpointer user_data)
{
  if (! get_page_text()) return;
  
  if (! cur_text_doc->linenums)
      document_set_line_numbers (cur_text_doc, TRUE);
   else
       document_set_line_numbers (cur_text_doc, FALSE);
}


void on_button_close (GtkWidget *wid, gpointer data)
{
  page_del_by_index (find_index_by_page (data));
}


void on_mni_html_default_template (GtkMenuItem *menuitem,
                                   gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *t = g_strconcat (
                          "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/1999/REC-html401-19991224/loose.dtd\">\n", 
                          "<html>\n",
                          "<head>\n",
                          "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n",
                          "<title></title>\n",
                          "</head>\n",
                          "<body>\n",
                          "</body>\n",
                          "</html>",
                          NULL);
                                       
  doc_insert_at_cursor (cur_text_doc, t);
  g_free (t);
}


void file_save_session_callback_widget (GtkWidget *widget, gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (user_data));

  GList *p = g_list_first (dox);
  GList *t = NULL;

  while (p)
        {
         t = add_recent_item_composed (t, (t_note_page*) p->data);
         p = g_list_next (p);
        }

  glist_save_to_file (t, filename);                       
  reload_sessions ();
}


void on_mni_file_save_session (GtkMenuItem *menuitem,
                               gpointer user_data)
{
  GtkWidget *file_selector = gtk_file_selection_new (_("Save the session as:"));

  gtk_file_selection_set_filename (file_selector, confile.sessions_dir);

  g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                    "clicked",
                    G_CALLBACK (file_save_session_callback_widget),
                    (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                             "clicked",
                             G_CALLBACK (gtk_widget_destroy),
                             (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->cancel_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  gtk_window_resize (file_selector, get_value (confile.screen_w, confile.saveopen_w), get_value (confile.screen_h, confile.saveopen_h));
  gtk_widget_show (file_selector);
}



void on_mni_session_file_open_activate (GtkMenuItem *menuitem,
                                        gpointer user_data)
{
  GtkFileSelection *file_selector = gtk_file_selection_new (_("Open a session file:"));

  g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                    "clicked",
                    G_CALLBACK (file_open_callback_widget),
                    (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->cancel_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  gtk_file_selection_set_filename (file_selector, confile.sessions_dir);
  gtk_window_resize (file_selector, get_value (confile.screen_w, confile.saveopen_w), get_value (confile.screen_h, confile.saveopen_h));
  gtk_widget_show (file_selector);
}


void on_mni_antispam (GtkMenuItem *menuitem, gpointer user_data)
{
  if (! get_page_text()) return;
 
  gchar *buf = doc_get_sel (cur_text_doc);
  if (! buf) return;

  gint c = g_utf8_strlen (buf, STRMAXLEN);
  GString *result = g_string_sized_new (16 * c);

  gint i;

  for (i = 0; i <= c; i++) 
       g_string_append_printf (result, "&#%d;", buf[i]);

  doc_rep_sel (cur_text_doc, result->str);

  g_free (buf);
  g_string_free (result, TRUE);
}


void on_mni_Functions_number_hex2dec (GtkMenuItem *menuitem,
                                      gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);
  if (! buf) return;

  gchar *t = g_strdup_printf ("%d", strtol (buf, NULL, 16));  

  doc_rep_sel (cur_text_doc, t);

  g_free (t);
  g_free (buf);
}


void on_mni_config_open (GtkMenuItem *menuitem,
                         gpointer user_data)
{
  cur_settings.selected_enc = ch_str (cur_settings.selected_enc, "UTF-8");
  open_file_std (confile.tea_rc);
}


void on_mni_open_in_browser (GtkMenuItem *menuitem,
                             gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *cmd;
  gchar *wn = gtk_widget_get_name (menuitem);

  if (g_ascii_strncasecmp (wn, "opera", 33) == 0)
     cmd = rep_all_s (confile.cmd_Opera, cur_text_doc->file_name);
  else     
  if (g_ascii_strncasecmp (wn, "firefox", 33) == 0)
     cmd = rep_all_s (confile.cmd_Firefox, cur_text_doc->file_name);
     //firefox -a firefox -remote openURL(<url>,new-tab)

  else
  if (g_ascii_strncasecmp (wn, "mozilla", 33) == 0)
     cmd = rep_all_s (confile.cmd_Mozilla, cur_text_doc->file_name);
  else
  if (g_ascii_strncasecmp (wn, "konqueror", 33) == 0)
     cmd = rep_all_s (confile.cmd_Konqueror, cur_text_doc->file_name);
  else
  if (g_ascii_strncasecmp (wn, "galeon", 33) == 0)
     cmd = rep_all_s (confile.cmd_Galeon, cur_text_doc->file_name);
  else
  if (g_ascii_strncasecmp (wn, "dillo",33) == 0)
     cmd = rep_all_s (confile.cmd_Dillo, cur_text_doc->file_name);
  else
  if (g_ascii_strncasecmp (wn, "elinks",33) == 0)
     cmd = rep_all_s (confile.cmd_ELinks, cur_text_doc->file_name);
  else
  if (g_ascii_strncasecmp (wn, "links",33) == 0)
     cmd = rep_all_s (confile.cmd_Links, cur_text_doc->file_name);
  else
  if (g_ascii_strncasecmp (wn, "lynx", 33) == 0)
     cmd = rep_all_s (confile.cmd_Lynx, cur_text_doc->file_name);
  else
  if (g_ascii_strncasecmp (wn, "epiphany", 33) == 0)
     cmd = rep_all_s (confile.cmd_Epiphany, cur_text_doc->file_name);
  else
  if (g_ascii_strncasecmp (wn, "Other browser", 33) == 0)
     cmd = rep_all_s (confile.cmd_Other, cur_text_doc->file_name);

  system (cmd);
  g_free(cmd);
}


void on_mni_utils_stats (GtkMenuItem *menuitem,
                         gpointer user_data)
{
  if (! get_page_text()) return;
  make_stats (cur_text_doc);
}


void on_mni_utils_add_encodings_activate (GtkMenuItem *menuitem,
                                          gpointer user_data)
{
  cur_settings.selected_enc = ch_str (cur_settings.selected_enc, "UTF-8");
  open_file_std (confile.iconv_file);
}


void on_mni_Functions_number_bin2dec (GtkMenuItem *menuitem, gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);
  if (! buf) return;
  
  gchar *t = g_strdup_printf ("%d", strtol (buf, NULL, 2)); 

  doc_rep_sel (cur_text_doc, t);

  g_free (t);
  g_free (buf);
}

                                       
void on_mni_Markup_nbsp (GtkMenuItem *menuitem,
                         gpointer user_data)
{
  if (! get_page_text()) return;
  doc_insert_at_cursor (cur_text_doc, "&nbsp;");
}


void on_mni_undo (GtkMenuItem *menuitem,
                  gpointer user_data)
{
  if (! get_page_text()) return;
  if (undomgr_can_undo (cur_text_doc->um))
      undomgr_undo (cur_text_doc->um);
}


void on_mni_redo (GtkMenuItem *menuitem,
                  gpointer user_data)
{
  if (! get_page_text()) return;
  if (undomgr_can_redo (cur_text_doc->um))
     undomgr_redo (cur_text_doc->um);
}


void ins_image_callback_widget (GtkWidget *widget,
                                gpointer user_data)
{
  gchar **files = gtk_file_selection_get_selections (user_data);
  gint c = -1;

  if (files)
     while (files [++c])
           {
            if (! g_file_test (files[c], G_FILE_TEST_IS_DIR))
               if (is_image (files[c]))
                  insert_image (files [c]);  
           }

   g_strfreev (files);
}


void on_mni_image_activate (GtkMenuItem *menuitem,
                            gpointer user_data)
{
  if (! get_page_text()) return;
  
  GtkFileSelection *file_selector = (GtkFileSelection *) gtk_file_selection_new (_("Select file"));
  gtk_file_selection_set_select_multiple (file_selector, TRUE);

  g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                    "clicked",
                    G_CALLBACK (ins_image_callback_widget),
                    (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->cancel_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  gtk_window_resize (file_selector, get_value (confile.screen_w, confile.saveopen_w), get_value (confile.screen_h, confile.saveopen_h));

  gchar *dir;

  if (g_file_test (cur_text_doc->file_name, G_FILE_TEST_EXISTS))
    {
     dir = g_strconcat (g_path_get_dirname (cur_text_doc->file_name), G_DIR_SEPARATOR_S, NULL);
     gtk_file_selection_set_filename (file_selector, dir);
     g_free (dir);
    }

  gtk_widget_show (file_selector);
}


void on_mni_html_enclose_link (GtkMenuItem* menuitem,
                              gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf) return;

  gchar *t = linkify_text (buf);
  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  free (t);
}


gboolean on_ent_search_key_press_event (GtkWidget *widget,
                                        GdkEventKey *event,
                                        gpointer user_data)
{
  gchar *s;

  if (event->keyval != GDK_Return)
     return FALSE;      

  if (get_page_text())
     {
      set_fam_text ("text to find");
      s = gtk_entry_get_text (ent_search);
      doc_search_f (cur_text_doc, s);
     }

  return FALSE;
}


void on_mni_func_strings_removeblanks (GtkMenuItem* menuitem,
                                       gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf) return;

  GList *temp = glist_from_string (buf);
  GList *temp2 = remove_blank_lines (temp);
  gchar *t = string_from_glist (temp2);

  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
  glist_strings_free (temp);
  glist_strings_free (temp2);
}


void on_mni_func_strings_enclose (GtkMenuItem* menuitem,
                                  gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf) return;

  set_fam_text ("<li>%s</li>");

  if (! strstr (gtk_entry_get_text (ent_search), "%s"))
     {
      g_free (buf);
      return;
     }

  GList *temp = glist_from_string (buf);

  temp = glist_repl (temp, gtk_entry_get_text (ent_search));
  gchar *t = string_from_glist (temp);
  

  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
  glist_strings_free (temp);
}


void on_mni_func_strings_rev (GtkMenuItem* menuitem,
                              gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf) return;

  GList *temp = glist_from_string (buf);
  temp = g_list_reverse (temp);
  gchar *t = string_from_glist (temp);
  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
  glist_strings_free (temp);
}


void on_mni_search_repall (GtkMenuItem* menuitem,
                          gpointer user_data)
{
  if (! get_page_text()) return;
  
  set_fam_text ("text to find~text to replace");
  gchar *s = gtk_entry_get_text (ent_search);

  if (! strstr (s, "~"))
     return; 
 
  gchar **a = g_strsplit (s, "~", -1);

  if (! a) return;

  gchar *x; 
  gchar *y;

  x = a[0];
  if (! x)
     {
      g_strfreev (a);
      return;
     } 

  if (! a[1])
     {
      g_strfreev (a);
      return;
     } 

  y = a[1];

  gchar *buf = doc_get_buf (cur_text_doc->text_buffer);
  gchar *z = str_replace_all (buf, x, y);
 
  gtk_text_buffer_set_text (cur_text_doc->text_buffer, z, -1);

  g_free (z);
  g_free (buf); 
  g_strfreev (a);
}


void on_mni_recent_activate (GtkMenuItem *menuitem,
                             gpointer user_data)
{
  execute_recent_item (gtk_widget_get_name (menuitem));
}


void on_mni_about_activate (GtkMenuItem *menuitem,
                            gpointer user_data)
{
  create_wnd_about ();
}


void show_save_as_dlg (int mode)
{
  gchar *dir;
  GtkFileSelection *file_selector = (GtkFileSelection *) gtk_file_selection_new (_("Save as..."));

  g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                    "clicked",
                    G_CALLBACK (file_save_as_callback_widget),
                    (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->cancel_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);


  if (mode == 0)
     {
      t_ppair *p = g_malloc (sizeof (t_ppair));
      p->a = file_selector;
      p->b = file_selector->main_vbox;
      GtkWidget *w = create_dialog_control_box (p, 1);
      g_free (p);
     }

  gtk_window_resize (file_selector, get_value (confile.screen_w, confile.saveopen_w), get_value (confile.screen_h, confile.saveopen_h));

  if (mode == 1)
     {
      cur_settings.selected_enc = ch_str (cur_settings.selected_enc, "UTF-8");
      gtk_file_selection_set_filename (file_selector, confile.templates_dir);
     }

  if (mode == 0)
     {
      if (cur_text_doc)
         {
          if (confile.use_def_save_dir)     
             gtk_file_selection_set_filename (file_selector, confile.def_save_dir);
          else
              if (g_file_test (cur_text_doc->file_name, G_FILE_TEST_EXISTS))
                 gtk_file_selection_set_filename (file_selector, cur_text_doc->file_name);
         }
     }

  if (mode == 2)
     gtk_file_selection_set_filename (file_selector, confile.snippets_dir);

  gtk_widget_show (file_selector);
}


void on_mni_file_save_as_template_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  show_save_as_dlg (1);
}


void on_mni_file_edit_at_cursor (GtkMenuItem* menuitem,
                              gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *w = get_c_url(cur_text_doc);
  handle_file (w, 1);
  g_free (w);
}


void on_mni_new_link_select (GtkMenuItem *menuitem,
                             gpointer user_data)
{
  handle_file (gtk_widget_get_name (menuitem), 0);
}


void add_link_item (const gchar *fname, const gchar *linkname)
{
  gchar *dir = g_path_get_dirname (fname);
  gchar *filename = create_full_path (linkname, dir);

  if (g_file_test (filename, G_FILE_TEST_EXISTS))
      mni_temp = new_menu_item (filename, mni_links_menu, on_mni_new_link_select);

  g_free (dir);
  g_free (filename);
}

//n.p. Defecation - Intention Superpassed - 2/3 Pure 
void scan_links (void)
{
  if (! get_page_text()) return;
  
  gchar *f;
  GtkTextIter match_start;
  GtkTextIter match_end;

  gtk_widget_destroy(mni_links_menu);
  mni_links_menu = new_menu_submenu (mni_links);
  mni_temp = new_menu_item (cur_text_doc->file_name, mni_links_menu, on_mni_new_link_select);
  gtk_text_buffer_get_start_iter (cur_text_doc->text_buffer, &match_start); 
  gtk_text_buffer_get_end_iter (cur_text_doc->text_buffer, &match_end); 

  while (gtk_text_iter_forward_search (&match_start, "ref=\"", GTK_TEXT_SEARCH_TEXT_ONLY, &match_start, &match_end, NULL))
       {
        match_start = match_end;
        if (gtk_text_iter_forward_find_char (&match_end, find_quote, NULL, NULL))   
           { 
            f = gtk_text_buffer_get_text (cur_text_doc->text_buffer, &match_start, &match_end, FALSE);
            add_link_item (cur_text_doc->file_name, f);
            g_free (f);
           }
        match_start = match_end;
      }

  gtk_text_buffer_get_start_iter (cur_text_doc->text_buffer, &match_start); 
  gtk_text_buffer_get_end_iter (cur_text_doc->text_buffer, &match_end); 

  while (gtk_text_iter_forward_search (&match_start, "REF=\"", GTK_TEXT_SEARCH_TEXT_ONLY, &match_start, &match_end, NULL))
       {
        match_start = match_end;
        if (gtk_text_iter_forward_find_char (&match_end, find_quote, NULL, NULL))   
           {
            f = gtk_text_buffer_get_text (cur_text_doc->text_buffer, &match_start, &match_end, FALSE);
            add_link_item (cur_text_doc->file_name, f);
            g_free (f);
           }
        match_start = match_end;
       }
}


void on_mni_get_links (GtkMenuItem* menuitem,
                       gpointer user_data)
{
  scan_links ();
  log_to_memo (_("Now look into Links menu ;)"), NULL, LM_ADVICE);
}


//n.p. Massive Attack - Karmacoma [album version]
void on_mni_co_select (GtkMenuItem *menuitem, gpointer user_data)
{
  if (! get_page_text()) return;
  
  if (! g_file_test (cur_text_doc->file_name, G_FILE_TEST_EXISTS))
     return;

  g_free (cur_settings.selected_enc);
  cur_settings.selected_enc = g_strdup (gtk_widget_get_name (menuitem));
  if (! doc_reload_text (cur_text_doc, cur_text_doc->file_name, cur_settings.selected_enc))
     log_to_memo (_("No, I can't!"), NULL, LM_ERROR);
  else
      cur_text_doc->encoding = g_strdup (cur_settings.selected_enc);
}


void on_mni_html_strip_tags (GtkMenuItem* menuitem,
                              gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);
  if (! buf) return;

  gchar *t = strip_html (buf);
  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
}


void on_mni_markup_align (GtkMenuItem *menuitem,
                                      gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *t;
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf)
     {
      if (strcmp (gtk_widget_get_name (menuitem), "Center") == 0)
          doc_insert_at_cursor (cur_text_doc,"<div align=\"center\"></div>");
      else
      if (strcmp (gtk_widget_get_name (menuitem), "Left") == 0)
          doc_insert_at_cursor (cur_text_doc,"<div align=\"left\"></div>");
      else
      if (strcmp (gtk_widget_get_name (menuitem), "Right") == 0)
          doc_insert_at_cursor (cur_text_doc,"<div align=\"right\"></div>");
      else
      if (strcmp (gtk_widget_get_name (menuitem), "Justify") == 0)
          doc_insert_at_cursor (cur_text_doc,"<div align=\"justify\"></div>");

      g_free (buf);
      doc_move_cursor_backw_middle_tags (cur_text_doc);  

      return;
     }
  else
     {  
      if (strcmp (gtk_widget_get_name (menuitem), "Center") == 0)
          t = g_strconcat ("<div align=\"center\">", buf, "</div>", NULL);
      else
      if (strcmp (gtk_widget_get_name (menuitem), "Left") == 0)
         t =  g_strconcat ("<div align=\"left\">", buf, "</div>", NULL);
      else
      if (strcmp (gtk_widget_get_name (menuitem), "Right") == 0)
         t =  g_strconcat ("<div align=\"right\">", buf, "</div>", NULL);
      else
      if (strcmp (gtk_widget_get_name (menuitem), "Justify") == 0)
         t =  g_strconcat ("<div align=\"justify\">", buf, "</div>", NULL);
      }
  
  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
}


void on_mni_html_make_table (GtkMenuItem* menuitem,
                              gpointer user_data)
{
  if (! get_page_text()) return;
  
  set_fam_text ("2~2"); 
  gchar *table = gtk_entry_get_text (ent_search);
  
  gchar **a = g_strsplit (table, "~", 3);

  if (! a)
     {
      g_free (table);
      return;
     }

  gint cols = 1;
  gint rows = 1;
  gint c;
  gint r;

  gchar *buf = strdup ("<table>\n");

  if (a[0])
     rows = strtol (a[0], NULL, 10);
  if (a[1])
     cols = strtol (a[1], NULL, 10);

  for (r = 1; r <= rows; r++)
      {
       buf = g_strconcat (buf, "<tr>\n", NULL);
       for (c = 1; c <= cols; c++)
            buf = g_strconcat (buf, "<td></td>\n", NULL);

       buf = g_strconcat (buf, "</tr>\n", NULL);
      }

  buf = g_strconcat (buf, "</table>", NULL);
  doc_insert_at_cursor (cur_text_doc, buf);

  g_strfreev (a);
  g_free (buf);
}


void on_mni_file_revert_to_saved (GtkMenuItem* menuitem,
                                  gpointer user_data)
{
  if (! get_page_text()) return;
  
  if (! g_file_test (cur_text_doc->file_name, G_FILE_TEST_EXISTS))
     return;

  if (! doc_reload_text (cur_text_doc, cur_text_doc->file_name, cur_text_doc->encoding))
    log_to_memo (_("I can't reload %s :("), cur_text_doc->file_name, LM_ERROR);
  else
      log_to_memo (_("%s is reloaded :)"), cur_text_doc->file_name, LM_NORMAL);
}


void on_mni_funx_ins_time (GtkMenuItem *menuitem, gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = get_time (confile.date_time);
  doc_insert_at_cursor (cur_text_doc, buf);
  g_free (buf);
}


void on_mni_goto_line (GtkMenuItem* menuitem,
                       gpointer user_data)
{
  if (! get_page_text()) return;
  set_fam_text ("0");
  doc_select_line (cur_text_doc, strtol (gtk_entry_get_text (ent_search), NULL, 10));
}


void on_mni_convert_ascii_2_html (GtkMenuItem *menuitem,
                                  gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);
  if (! buf) return;

  gchar *t = g_markup_escape_text (buf, strlen (buf));
  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
}


void on_mni_func_filter_exclude_from_list (GtkMenuItem* menuitem,
                                           gpointer user_data)
{
  if (! get_page_text()) return;
  
  set_fam_text ("Chirurgus mente prius et oculis agat, quam armata manu");
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf) return;

  GList *temp = glist_from_string (buf);
  GList *temp2 = filter_exclude_from_list (temp, gtk_entry_get_text (ent_search));
  gchar *t = string_from_glist (temp2);

  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
  glist_strings_free (temp);
  glist_strings_free (temp2);
}


void on_mni_func_filter_antiexclude_from_list (GtkMenuItem* menuitem,
                                           gpointer user_data)
{
  if (! get_page_text()) return;
  
  set_fam_text ("Tantum possumus, quantum scimus.");  
  gchar *s = gtk_entry_get_text (ent_search);
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf) return;

  GList *temp = glist_from_string (buf);
  GList *temp2 = filter_antiexclude_from_list (temp, s);
  gchar *t = string_from_glist (temp2);

  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
  glist_strings_free (temp);
  glist_strings_free (temp2);
}


GList *common_temp_list = NULL;

void on_mni_html_calc_weight (GtkMenuItem *menuitem,
                              gpointer user_data)
{
  if (! get_page_text()) return;
  
  if (! g_file_test (cur_text_doc->file_name, G_FILE_TEST_EXISTS))
     return;

  common_temp_list = NULL;
  
  gchar *f;
  GtkTextIter match_start;
  GtkTextIter match_end;

  gtk_widget_destroy(mni_links_menu);
  mni_links_menu = new_menu_submenu (mni_links);
  mni_temp = new_menu_item (cur_text_doc->file_name, mni_links_menu, on_mni_new_link_select);
  gtk_text_buffer_get_start_iter (cur_text_doc->text_buffer, &match_start); 
  gtk_text_buffer_get_end_iter (cur_text_doc->text_buffer, &match_end); 

  while (gtk_text_iter_forward_search (&match_start, "src=\"", GTK_TEXT_SEARCH_TEXT_ONLY, &match_start, &match_end, NULL))
       {
        match_start = match_end;
        if (gtk_text_iter_forward_find_char (&match_end, find_quote, NULL, NULL))   
           {
            f = gtk_text_buffer_get_text (cur_text_doc->text_buffer, &match_start, &match_end, FALSE);
            if ( g_file_test (f, G_FILE_TEST_EXISTS)) 
                 common_temp_list = g_list_append (common_temp_list, f);   
           } 
        match_start = match_end;
       }

  while (gtk_text_iter_forward_search (&match_start, "SRC=\"", GTK_TEXT_SEARCH_TEXT_ONLY, &match_start, &match_end, NULL))
       {
        match_start = match_end;
        if (gtk_text_iter_forward_find_char (&match_end, find_quote, NULL, NULL))   
           {
            f = gtk_text_buffer_get_text (cur_text_doc->text_buffer, &match_start, &match_end, FALSE);
            if ( g_file_test (f, G_FILE_TEST_EXISTS)) 
               common_temp_list = g_list_append (common_temp_list, f);
           } 
        match_start = match_end;
       }

  gint bytes_total = 0;
  gchar *t = NULL;
  gint x;

  GList *p = g_list_first (common_temp_list);

  while (p)
        {
         x = get_file_size ((gchar *) p->data);
         bytes_total = bytes_total + x;
         t = g_strdup_printf (_("%s = %d Kb"), (gchar *) p->data, (gint) x/1024);
         log_to_memo (t, NULL, LM_NORMAL);
         g_free (t);
         p = g_list_next (p);
        }

  x = get_file_size (cur_text_doc->file_name);
  bytes_total = bytes_total + x;
  t = g_strdup_printf (_("%s = %d Kb"), cur_text_doc->file_name, (gint) x/1024);
  log_to_memo (t, NULL, LM_NORMAL);
  g_free (t);     

  glist_strings_free (common_temp_list);

  t = g_strdup_printf (_("Total weight for %s is %d Kb"), cur_text_doc->file_name, bytes_total/1024);
  log_to_memo (t, NULL, LM_NORMAL);
  g_free (t);
}


void on_mni_get_src (GtkMenuItem* menuitem,
                     gpointer user_data)
{
  if (! get_page_text()) return;
  
  if (! g_file_test (cur_text_doc->file_name, G_FILE_TEST_EXISTS))
     return;

  gchar *f;
  GtkTextIter match_start;
  GtkTextIter match_end;

  gtk_widget_destroy(mni_links_menu);
  mni_links_menu = new_menu_submenu (mni_links);
  mni_temp = new_menu_item (cur_text_doc->file_name, mni_links_menu, on_mni_new_link_select);
  gtk_text_buffer_get_start_iter (cur_text_doc->text_buffer, &match_start); 
  gtk_text_buffer_get_end_iter (cur_text_doc->text_buffer, &match_end); 

  while (gtk_text_iter_forward_search (&match_start, "src=\"", GTK_TEXT_SEARCH_TEXT_ONLY, &match_start, &match_end, NULL))
       {
        match_start = match_end;
        if (gtk_text_iter_forward_find_char (&match_end, find_quote, NULL, NULL))   
           {
            f = gtk_text_buffer_get_text (cur_text_doc->text_buffer, &match_start, &match_end, FALSE);
            add_link_item (cur_text_doc->file_name, f);
            g_free (f);
           }
        match_start = match_end;
       }

  while (gtk_text_iter_forward_search (&match_start, "SRC=\"", GTK_TEXT_SEARCH_TEXT_ONLY, &match_start, &match_end, NULL))
       {
        match_start = match_end;
        if (gtk_text_iter_forward_find_char (&match_end, find_quote, NULL, NULL))   
           {
            f = gtk_text_buffer_get_text (cur_text_doc->text_buffer, &match_start, &match_end, FALSE);
            add_link_item (cur_text_doc->file_name, f);
            g_free (f); 
           }
        match_start = match_end;
       }

  log_to_memo (_("Now look into Links menu ;)"), NULL, LM_ADVICE);
}


void select_color_cb (GtkWidget *widget,
                      gpointer user_data)
{
  GdkColor color;
  GtkColorSelection *colorsel;
  colorsel = GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (dlg_colorsel)->colorsel);

  gtk_color_selection_get_current_color (colorsel, &color);
  //gchar *s = g_strdup_printf("\"#%02x%02x%02x\"", color->red / 256, color->green / 256, color->blue / 256);
  //gchar *s = g_strdup_printf("#%02x%02x%02x", color.red / 256, color.green / 256, color.blue / 256);
  gchar *s = g_strdup_printf("#%02x%02x%02x", color.red / 256, color.green / 256, color.blue / 256);
  gchar *cm;
  gchar *t;
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf)
     doc_insert_at_cursor (cur_text_doc, s);
  else
      {
       //gchar *t = g_strconcat ("<font color=\"", s, "\">", buf, "</font>", NULL);
       //gchar *t = g_strdup_printf ("<span style=\"color:%s;\">%s</span>", s, buf);
      
       gchar *cm = g_strdup (confile.fmt_color_function);

       if (strstr (cm, "@color"))
          {
           t = g_str_replace (cm, "@color", s);
           //g_free (cm);
           //cm = t;
           cm = ch_str (cm, t);
          }

       if (strstr (cm, "@text"))
          {
           t = g_str_replace (cm, "@text", buf);
           cm = ch_str (cm, t);
           //g_free (cm);
           //cm = t;
          }


       //gchar *t = g_strdup_printf (confile.fmt_color_function, s, buf);
       //doc_rep_sel (cur_text_doc, t);
       doc_rep_sel (cur_text_doc, cm);
       g_free (cm);
//       g_free (t);
       g_free (buf);
      }

  g_free (s);
}


void on_mni_markup_select_color_activate (GtkMenuItem *menuitem,
                                         gpointer user_data)
{
  if (! get_page_text()) return;
  
  dlg_colorsel = gtk_color_selection_dialog_new (_("Select a color"));

  g_signal_connect (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg_colorsel)->ok_button),
                    "clicked",
                    G_CALLBACK (select_color_cb),
                    (gpointer) dlg_colorsel);
  
  g_signal_connect_swapped (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg_colorsel)->ok_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) dlg_colorsel);

  g_signal_connect_swapped (GTK_OBJECT (GTK_COLOR_SELECTION_DIALOG (dlg_colorsel)->cancel_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) dlg_colorsel);

  gtk_widget_show (dlg_colorsel);
}


void on_mni_file_backup (GtkMenuItem* menuitem,
                       gpointer user_data)
{
  if (! get_page_text()) return;
  
  if (g_file_test (cur_text_doc->file_name, G_FILE_TEST_EXISTS))
      do_backup (cur_text_doc->file_name, FALSE);
}


void on_mni_file_add_bookmark (GtkMenuItem* menuitem,
                               gpointer user_data)
{
  if (! get_page_text()) return;

  bookmarks = add_recent_item_composed (bookmarks, cur_text_doc);
  gchar *filename = g_strconcat (confile.tea_main_dir, G_DIR_SEPARATOR_S, "tea_bmx", NULL);
  glist_save_to_file (bookmarks, filename);
  g_free (filename);
  bmx_reload ();
}


void on_mni_file_open_bookmark (GtkMenuItem* menuitem,
                               gpointer user_data)
{
  g_free (cur_settings.selected_enc);
  cur_settings.selected_enc = g_strdup ("UTF-8");
  open_file_std (confile.bmx_file);
}


void on_mni_real_recent_activate (GtkMenuItem *menuitem,
                                  gpointer user_data)
{
  execute_recent_item (gtk_widget_get_name (menuitem));

  GList *p = find_item_by_data (recent_list, gtk_widget_get_name (menuitem));
  if (p)
     {
      g_free (p->data);
      recent_list = g_list_delete_link (recent_list, p);
      update_recent_list_menu (FALSE);
     }
}


void on_mni_trim_left (GtkMenuItem *menuitem,
                       gpointer user_data)
{
  if (! get_page_text()) return;
  set_fam_text ("1");
  gchar *s = gtk_entry_get_text (ent_search);
  
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf) return;
  
  gint i = strtol (s, NULL, 10);

  GList *temp = glist_from_string (buf);

  temp = glist_trim_l (temp, i);
  gchar *t = string_from_glist (temp);
  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
  glist_strings_free (temp);
}


void on_mni_trim_r_on_char (GtkMenuItem *menuitem,
                           gpointer user_data)
{
  if (! get_page_text()) return;

  set_fam_text ("a");
  gchar *s = gtk_entry_get_text (ent_search);
 
  gchar *buf = doc_get_sel (cur_text_doc);
  if (! buf) return;

  gchar c = s[0];

  GList *temp = glist_from_string (buf);
  temp = glist_trim_r_by_char (temp, c);
  gchar *t = string_from_glist (temp);

  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
  glist_strings_free (temp);
}


void on_mni_trim_l_on_char (GtkMenuItem *menuitem,
                           gpointer user_data)
{
  if (! get_page_text()) return;

  set_fam_text ("1"); 
  gchar *s = gtk_entry_get_text (ent_search);
  
  gchar *buf = doc_get_sel (cur_text_doc);
  if (! buf) return;

  gchar c = s[0];

  GList *temp = glist_from_string (buf);
  temp = glist_trim_r_by_char (temp, c);
  gchar *t = string_from_glist (temp);

  doc_rep_sel (cur_text_doc, t);
  g_free (buf);
  g_free (t);
  glist_strings_free (temp);
}


void on_mni_refresh_hl (GtkMenuItem *menuitem,
                        gpointer user_data)
{
  if (! get_page_text()) return;
  apply_hl (cur_text_doc);
}


void on_mni_make_numbering (GtkMenuItem *menuitem,
                            gpointer user_data)
{
  if (! get_page_text()) return;
  set_fam_text ("1~5~10");
  gchar *s = gtk_entry_get_text (ent_search);
  
  gchar *buf = doc_get_sel (cur_text_doc);
  if (buf) //exit if selected
     {
      g_free (buf);
      return;
     }

  gchar **a = g_strsplit (s, "~", 4);

  if (! a) return;

  gint c;
  gint x = 0;
  gint y = 0;
  gint step = 1;

  if (a[0])
     {
      x = strtol (a[0], NULL, 10);
      if (a[1])
         {
          y = strtol (a[1], NULL, 10);
          if (a[2])
              step = strtol (a[2], NULL, 10);
         }  
     }

  gchar *counter = NULL;
  GList *list = NULL;

  for (c = x; c <= y; (c++))
      {
       counter = g_strdup_printf ("%d", (c * step));
       list = g_list_append (list, counter);
      }

  gchar *t = string_from_glist (list);
  doc_insert_at_cursor (cur_text_doc, t);

  g_free (t);
  glist_strings_free (list);
}


void on_mni_nav_block_start (GtkMenuItem *menuitem,
                             gpointer user_data)
{
  if (! get_page_text()) return;

  GtkTextIter iter;
  gint c = 0;

  GtkTextMark *m = gtk_text_buffer_get_insert (cur_text_doc->text_buffer); 
  gtk_text_buffer_get_iter_at_mark (cur_text_doc->text_buffer, &iter, m);

  while (c != -1)
        {
         if (gtk_text_iter_backward_char (&iter))
         if (gtk_text_iter_get_char (&iter) == '}')
            c++;
         else
             if (gtk_text_iter_get_char (&iter) == '{')
                {
                 c--;
                 if (c == -1)
                 if (gtk_text_iter_forward_char (&iter))
                    {
                     gtk_text_buffer_place_cursor (cur_text_doc->text_buffer, &iter );
                     gtk_text_view_scroll_to_iter (cur_text_doc->text_view, &iter, 0.0, FALSE, 0.0, 0.0 );
                    }
                }
        }
}


void on_mni_nav_block_end (GtkMenuItem *menuitem,
                             gpointer user_data)
{
  if (! get_page_text()) return;

  GtkTextIter iter;
  gint c = 0;

  GtkTextMark *m = gtk_text_buffer_get_insert (cur_text_doc->text_buffer); 
  gtk_text_buffer_get_iter_at_mark (cur_text_doc->text_buffer, &iter, m);

  while (c != -1)
        {
         if (gtk_text_iter_forward_char (&iter))
         if (gtk_text_iter_get_char (&iter) == '{')
            c++;
            else
                if (gtk_text_iter_get_char (&iter) == '}')
                   {
                    c--;
                    if (c == -1)
                       {
                        gtk_text_buffer_place_cursor (cur_text_doc->text_buffer, &iter );
                        gtk_text_view_scroll_to_iter (cur_text_doc->text_view, &iter, 0.0, FALSE, 0.0, 0.0 );
                       }
                   }
        }
}


void on_mni_open_hotkeys (GtkMenuItem *menuitem,
                          gpointer user_data)
{
  //g_free (cur_settings.selected_enc);
  //cur_settings.selected_enc = g_strdup ("UTF-8");
  cur_settings.selected_enc = ch_str (cur_settings.selected_enc, "UTF-8");
  open_file_std (confile.tea_hotkeys);
}


enum
{
  TARGET_URILIST
};


static const GtkTargetEntry drag_types[] = {
                                            {"text/uri-list", 0, TARGET_URILIST}
                                           };

static gint inner_dnd_switch = 0;

void open_files_list (const gchar *s)
{
  GList *l = glist_from_string (s);
  if (! l)
     return;

  gchar *f;
  
  GList *t = g_list_first (l);
  while (t)
        {
         g_free (cur_settings.selected_enc);

         if (confile.do_enc_autodetect)
            cur_settings.selected_enc = g_strdup ("autodetect"); 
         else
             cur_settings.selected_enc = g_strdup ("UTF-8");

         gchar *x = strchr ((gchar *) t->data, '/'); 
         if (x)
            f = g_strdup (x);
          else 
              f = g_strdup ((gchar *) t->data);
       
         if (f)
            {
             g_strchomp (f);
             
             if (is_image (f))
                {
                 if (get_page_text ())   
                    insert_image (f); 
                }
             else
                 open_file_std (f);

             g_free (f);
            }

         t = g_list_next (t);
        }
   glist_strings_free (l);
}


void tea_win_on_drag_data_cb (GtkWidget *widget,
                              GdkDragContext *context,
                              gint x, gint y, GtkSelectionData *data, 
                              guint info, guint time) 
{
  if (inner_dnd_switch == 1)
     {
      inner_dnd_switch = 0;
      return;
     } 

  gchar *filename;
  inner_dnd_switch++;   
 
  if ((data->length == 0) || (data->format != 8) || (info != TARGET_URILIST)) 
     {
      gtk_drag_finish (context, FALSE, TRUE, time);
      return;
     }

  filename = g_strndup (data->data, data->length);
  //if (info == TARGET_URILIST)
  open_files_list (filename);  

  gtk_drag_finish (context, TRUE, TRUE, time);
  g_free (filename);
}


void set_dnd_accept (GtkWidget *widget)
{
  gtk_drag_dest_set (widget, (GTK_DEST_DEFAULT_ALL),
                     drag_types, 1,
                    (GDK_ACTION_DEFAULT | GDK_ACTION_COPY | GDK_ACTION_MOVE ));

  g_signal_connect (G_OBJECT (widget), "drag_data_received", G_CALLBACK (tea_win_on_drag_data_cb), widget);
}


GtkWidget* create_wnd_enc_query (gchar *f)
{
  GtkWidget *wnd_enc_query;
  GtkWidget *mn_encodings;
  
  wnd_enc_query = gtk_dialog_new_with_buttons ("?",
                                               GTK_WINDOW (tea_main_window),
                                               GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                               GTK_STOCK_OK,
                                               GTK_RESPONSE_ACCEPT,
                                               GTK_STOCK_CANCEL,
                                               GTK_RESPONSE_REJECT,
                                               NULL);

  gchar *t = g_strconcat (_("Please select the codepage for "), f, NULL);
  GtkWidget *label = gtk_label_new (t);
  g_free (t); 
  gtk_widget_show (label);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(wnd_enc_query)->vbox), label, FALSE, FALSE, 0);
  
  GtkWidget *mni_enc = gtk_option_menu_new ();
  GtkWidget *mni_m = gtk_menu_new ();

  gtk_widget_show (mni_enc);
  gtk_option_menu_set_menu (mni_enc, mni_m);

  g_free (cur_settings.selected_enc);

  if (confile.do_enc_autodetect)
       cur_settings.selected_enc = g_strdup ("autodetect");
  else
      cur_settings.selected_enc = g_strdup ("UTF-8");

  GList *p = g_list_first (confile.iconv_encs);
  while (p)
       {
        mni_temp = new_menu_item ((gchar*) p->data, mni_m, on_mni_new_file_open_select_enc);
        p = g_list_next (p);
       }

  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(wnd_enc_query)->vbox), mni_enc, FALSE, FALSE, 0);
  gtk_option_menu_set_history (mni_enc, 0);

  return wnd_enc_query;
}


t_note_page* query_enc (gchar *f)
{
  t_note_page *t = NULL; 
  
  GtkWidget *w = create_wnd_enc_query (f);
  
  if (! w)
     return NULL;

  if (gtk_dialog_run (GTK_DIALOG (w)) == GTK_RESPONSE_ACCEPT)
     {
      t = open_file_std (f);
      gtk_widget_destroy (w);;
      return t;
     }
  else
      {
       gtk_widget_destroy (w);;
       return NULL;
      } 
}


gboolean on_tea_window_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  ui_done ();
  return FALSE;
}


void on_mni_snippet_click (GtkMenuItem *menuitem,
                           gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *f = g_strconcat (confile.snippets_dir, gtk_widget_get_name (menuitem), NULL);

  gsize length;
  GError *error;
  gchar *buf;
  gchar *t;
 
  if (! g_file_get_contents (f, &buf, &length, &error))
     return;
  else
      if (! g_utf8_validate (buf, -1, NULL))
         {
          g_free (buf);
          g_free (f);
          return;
         }
 
  gchar *sel = doc_get_sel (cur_text_doc);
  
  if (sel && strstr (sel, "%s"))
     {
      g_free (f);
      g_free (buf);
      return;
     }  

  if (sel)
     {
      if (strstr (buf, "%s"))
        {
         t = rep_all_s (buf, sel);
         doc_rep_sel (cur_text_doc, t); 
         g_free (t);
         g_free (sel);
        }
     }
     else
         doc_insert_at_cursor (cur_text_doc, buf);
  
  g_free (f);
  g_free (buf);
}


void on_mni_snippet_file_open_activate (GtkMenuItem *menuitem,
                                        gpointer user_data)
{
  GtkFileSelection *file_selector = gtk_file_selection_new (_("Open file:"));

  g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                    "clicked",
                    G_CALLBACK (file_open_callback_widget),
                    (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->cancel_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  gtk_file_selection_set_filename (file_selector, confile.snippets_dir);
  gtk_window_resize (file_selector, get_value (confile.screen_w, confile.saveopen_w), get_value (confile.screen_h, confile.saveopen_h));
  gtk_widget_show (file_selector);
}


void on_mni_file_save_as_snippet_activate (GtkMenuItem *menuitem, gpointer user_data)
{
  show_save_as_dlg (2);
}


static GList *gl_menu_item_names;

static void lookup_widget_cb_2 (GtkWidget *widget, gpointer data)
{
  if (! widget) return;
   
  gchar *s = gtk_widget_get_name (widget);
  if (strcmp ("-x-", s) != 0)
     {
      gl_menu_item_names = g_list_append (gl_menu_item_names, s);

      if (GTK_IS_MENU_ITEM (widget))
         lookup_widget2 (gtk_menu_item_get_submenu (widget), data);      
     }   
}


static GtkWidget* lookup_widget2 (GtkContainer *widget, const gchar *widget_name)
{
  if (widget)
     gtk_container_foreach (widget, lookup_widget_cb_2, NULL);
  return NULL;
}


void on_mni_dump_menu (GtkMenuItem *menuitem, gpointer user_data)
{
  cur_text_doc = doc_clear_new ();
  if (! get_page_text()) return;
   
  g_list_free (gl_menu_item_names);
  lookup_widget2 (GTK_CONTAINER (menubar1), NULL);
   
  gchar *t = string_from_glist (gl_menu_item_names); 

  doc_insert_at_cursor (cur_text_doc, t);
  g_free (t);
  g_list_free (gl_menu_item_names);
}


GtkWidget* create_wnd_imgviewer (gchar *f)
{
  GtkWidget *wnd_imgviewer = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  g_signal_connect ((gpointer) wnd_imgviewer, "destroy_event",
                    G_CALLBACK (gtk_false),
                    NULL);

  GtkWidget *image = gtk_image_new_from_file (f);
  gtk_container_add (GTK_CONTAINER (wnd_imgviewer), image);
  gtk_widget_show (image);
  gtk_widget_show (wnd_imgviewer);
  gchar *t = g_path_get_basename (f);
  gtk_window_set_title (wnd_imgviewer, t);
  g_free (t);
  return wnd_imgviewer;
}


void on_mni_nav_focus_to_famous (GtkMenuItem* menuitem,
                                gpointer user_data)
{
  gtk_widget_grab_focus (ent_search);    
}


void on_mni_nav_focus_to_text (GtkMenuItem* menuitem,
                               gpointer user_data)
{
  if (! get_page_text()) return;
  gtk_widget_grab_focus (cur_text_doc->text_view);    
}


void on_mni_tools_unitaz_sort (GtkMenuItem* menuitem,
                               gpointer user_data)
{
  if (! get_page_text()) return;
  run_unitaz (cur_text_doc, 0, FALSE);
}


void on_mni_tools_unitaz_sort_by_count (GtkMenuItem *menuitem,
                                        gpointer user_data)
{
  if (! get_page_text()) return;
  run_unitaz (cur_text_doc, 1, FALSE);
}


void on_mni_tools_unitaz_plain (GtkMenuItem* menuitem,
                                gpointer user_data)
{
  if (! get_page_text()) return;
  run_unitaz (cur_text_doc, -1);
}


void on_mni_str_kill_dups (GtkMenuItem *menuitem,
                           gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *buf = doc_get_sel (cur_text_doc);
  if (! buf) return;

  gchar *t = str_kill_dups (buf);
  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
}


void on_mni_func_strings_sort (GtkMenuItem* menuitem,
                              gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf) return;

  GList *temp = glist_from_string (buf);
  temp = glist_strings_sort (temp);
  gchar *t = string_from_glist (temp);

  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
  glist_strings_free (temp);
}


void on_mni_extract_words (GtkMenuItem* menuitem,
                           gpointer user_data)
{
  if (! get_page_text()) return;
  run_extract_words (cur_text_doc);
}


void on_mni_edit_copy_all (GtkMenuItem *menuitem,
                           gpointer user_data)
{
  if (! get_page_text()) return;
  
  GtkClipboard *c = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
  if (! c)
     return; 

  gchar *buf = doc_get_buf (cur_text_doc->text_buffer);

  if (buf)
     gtk_clipboard_set_text (c, buf, -1);

  g_free (buf);
}


void on_mni_edit_copy_current_url (GtkMenuItem *menuitem,
                                   gpointer user_data)
{
  if (! get_page_text()) return;
  
  GtkClipboard *c = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
  if (! c)
     return; 

  gchar *buf = get_c_url_pure (cur_text_doc);

  if (buf)
     gtk_clipboard_set_text (c, buf, -1);

  g_free (buf);
}


void on_mni_edit_replace_with_clipboard (GtkMenuItem *menuitem,
                                         gpointer user_data)
{
  if (! get_page_text()) return;
  
  GtkClipboard *c = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
  if (! c)
     return; 

  if (! gtk_clipboard_wait_is_text_available (c))
     return;

  gchar *buf = doc_get_sel (cur_text_doc);
  if (! buf) return;

  gchar *t = gtk_clipboard_wait_for_text (c); 
 
  doc_rep_sel (cur_text_doc, t);
  gtk_clipboard_set_text (c, buf, -1);

  g_free (buf);
  g_free (t);
}


void on_mni_count_string_list (GtkMenuItem *menuitem,
                               gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *buf = doc_get_sel (cur_text_doc);
  if (! buf) return;

  set_fam_text ("%d.)%s");  
  
  gchar *t = glist_enum (buf, gtk_entry_get_text (ent_search));

  if (t)
     doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);

}


void on_mni_paste_to_new (GtkMenuItem *menuitem,
                          gpointer user_data)
{
  GtkClipboard *c = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
  if (! c)
     return; 

  if (! gtk_clipboard_wait_is_text_available (c))
     return;

  cur_text_doc = doc_clear_new ();
  gtk_window_set_title (GTK_WINDOW (tea_main_window), cur_text_doc->file_name);
  gtk_widget_grab_focus (cur_text_doc->text_view);

  gchar *t = gtk_clipboard_wait_for_text (c); 
  doc_insert_at_cursor (cur_text_doc, t);
  g_free (t); 
}


void on_mni_copy_to_new (GtkMenuItem *menuitem,
                         gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *t = doc_get_sel (cur_text_doc);
  if (! t)
     return;  
 
  cur_text_doc = doc_clear_new ();
  gtk_window_set_title (GTK_WINDOW (tea_main_window), cur_text_doc->file_name);
  gtk_widget_grab_focus (cur_text_doc->text_view);

  doc_insert_at_cursor (cur_text_doc, t);
  g_free (t); 
}


void on_mni_cut_to_new (GtkMenuItem *menuitem,
                        gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *t = doc_get_sel (cur_text_doc);
  if (! t)
     return;  
 
  doc_rep_sel (cur_text_doc, "");
  doc_ins_to_new (t);
  g_free (t); 
}


void on_mni_shuffle_strings (GtkMenuItem *menuitem,
                             gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf) return;

  GList *temp = glist_from_string (buf);
  temp = glist_shuffle (temp);
  gchar *t = string_from_glist (temp);
  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
  glist_strings_free (temp);
}


void on_mni_tools_unitaz_sort_caseinsens (GtkMenuItem *menuitem,
                                          gpointer user_data)
{
  if (! get_page_text()) return;
  run_unitaz (cur_text_doc, 0, TRUE);
}


void on_mni_tools_unitaz_sort_by_count_caseinsens (GtkMenuItem *menuitem,
                                                   gpointer user_data)
{
  if (! get_page_text()) return;
  run_unitaz (cur_text_doc, 1, TRUE);
}


void on_mni_cure (GtkMenuItem *menuitem,
                  gpointer user_data)
{
  log_to_memo (_("That is the question!"), NULL, LM_ERROR);  
}


void on_mni_tabs_to_spaces (GtkMenuItem *menuitem,
                            gpointer user_data)
{
  if (! get_page_text()) return;
  set_fam_text ("1");
  doc_tabs_to_spaces (cur_text_doc, strtol (gtk_entry_get_text (ent_search), NULL, 10));  
}


void on_mni_spaces_to_tabs (GtkMenuItem *menuitem,
                            gpointer user_data)
{
  if (! get_page_text()) return;
  set_fam_text ("2");
  doc_spaces_to_tabs (cur_text_doc, strtol (gtk_entry_get_text (ent_search), NULL, 10));  
}


void on_mni_edit_delete_current_line (GtkMenuItem *menuitem,
                                      gpointer user_data)
{
  if (! get_page_text()) return;
  
  GtkTextIter ittemp;
  GtkTextIter itend;

  GtkTextMark *mark = gtk_text_buffer_get_insert (cur_text_doc->text_buffer);
 
  gtk_text_buffer_get_iter_at_mark (cur_text_doc->text_buffer, &ittemp, mark);

  itend = ittemp;

  if (gtk_text_iter_forward_to_line_end (&itend))
     while (! gtk_text_iter_starts_line (&ittemp))
           gtk_text_iter_backward_char (&ittemp); 

  if (gtk_text_iter_forward_char (&itend));
     gtk_text_buffer_delete (cur_text_doc->text_buffer, &ittemp, &itend);

}


void on_mni_Markup_comment (GtkMenuItem *menuitem,
                           gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *temp = NULL;
  gchar *buf = doc_get_sel (cur_text_doc);

  if (is_markup (cur_text_doc->file_name))
    {
     if (! buf)
        doc_insert_at_cursor (cur_text_doc, "<!-- -->");
     else
         {
          temp = g_strconcat ("<!-- ", buf, " -->", NULL);
          doc_rep_sel (cur_text_doc, temp);
          g_free (temp);
         }
  
     g_free (buf);
     return;
    }

  if (is_c (cur_text_doc->file_name) || is_css (cur_text_doc->file_name))
     {
      if (! buf)
        doc_insert_at_cursor (cur_text_doc, "/* */");
      else
          {
           temp = g_strconcat ("/*", buf, "*/", NULL);
           doc_rep_sel (cur_text_doc, temp);
           g_free (temp);
          }

      g_free (buf);
      return; 
     } 
  
  if (is_pascal (cur_text_doc->file_name))
     {
      if (! buf)
         doc_insert_at_cursor (cur_text_doc, "{ }");
      else
          {
           temp = g_strconcat ("{", buf, "}", NULL);
           doc_rep_sel (cur_text_doc, temp);
           g_free (temp);
          }

      g_free (buf);
      return; 
     } 
}


void on_mni_paste (GtkMenuItem *menuitem,
                   gpointer user_data)
{
  if (! get_page_text()) return;
  
  GtkClipboard *c = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
  if (! c)
     return; 

  if (! gtk_clipboard_wait_is_text_available (c))
     return;

  gchar *t = gtk_clipboard_wait_for_text (c); 
  doc_insert_at_cursor (cur_text_doc, t);
  g_free (t); 
}


void on_mni_edit_copy (GtkMenuItem *menuitem,
                       gpointer user_data)
{
  if (! get_page_text()) return;
  
  GtkClipboard *c = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
  if (! c)
     return; 

  gchar *buf = doc_get_sel  (cur_text_doc);

  if (buf)
      gtk_clipboard_set_text (c, buf, -1);

  g_free (buf);
}


void on_mni_edit_cut (GtkMenuItem *menuitem,
                       gpointer user_data)
{
  if (! get_page_text()) return;
  
  GtkClipboard *c = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
  if (! c)
     return; 

  gchar *buf = doc_get_sel  (cur_text_doc);

  if (buf)
     {
      gtk_clipboard_set_text (c, buf, -1);
      doc_rep_sel (cur_text_doc, "");
     }
      
  g_free (buf);
}


void on_mni_edit_delete (GtkMenuItem *menuitem,
                         gpointer user_data)
{
  if (! get_page_text()) return;
  doc_rep_sel (cur_text_doc, "");
}


void on_mni_edit_select_all (GtkMenuItem *menuitem,
                             gpointer user_data)
{
  if (! get_page_text()) return;

  GtkTextIter start_iter;
  GtkTextIter end_iter;

  gtk_text_buffer_get_start_iter (cur_text_doc->text_buffer, &start_iter);
  gtk_text_buffer_get_end_iter (cur_text_doc->text_buffer, &end_iter);

  gtk_text_buffer_move_mark_by_name (cur_text_doc->text_buffer,
                                     "insert",
                                     &start_iter);

  gtk_text_buffer_move_mark_by_name (cur_text_doc->text_buffer,
                                     "selection_bound",
                                     &end_iter);
}


void on_mni_sort_case_insensetive (GtkMenuItem *menuitem,
                                   gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf) return;
  
  GList *temp = glist_from_string (buf);
  temp = sort_list_case_insensetive (temp);
  gchar *t = string_from_glist (temp);

  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
  glist_strings_free (temp);
}


void on_mni_kill_formatting (GtkMenuItem *menuitem,
                             gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);

  if (buf)
     {
      gchar *temp = kill_formatting (buf, -1);
      doc_rep_sel (cur_text_doc, temp);
      g_free (buf);
      g_free (temp);
     }
}


void on_mni_wrap_raw (GtkMenuItem *menuitem,
                      gpointer user_data)
{
  if (! get_page_text()) return;

  set_fam_text ("72");
  
  gchar *buf = doc_get_sel (cur_text_doc);
  if (buf)
     {
      gchar *temp = wrap_raw (buf, strtol (gtk_entry_get_text (ent_search), NULL, 10));
      doc_rep_sel (cur_text_doc, temp);
      g_free (buf);
      g_free (temp);
     }
}


void on_mni_wrap_on_spaces (GtkMenuItem *menuitem,
                            gpointer user_data)
{
  if (! get_page_text()) return;

  set_fam_text ("72");
  
  gchar *buf = doc_get_sel (cur_text_doc);
  if (buf)
     {
      wrap_on_spaces (buf, strtol (gtk_entry_get_text (ent_search), NULL, 10));
      doc_rep_sel (cur_text_doc, buf);
      g_free (buf);
     }
}


void on_doit_button (GtkButton *button,
                     gpointer user_data)
{
  if (get_page_text())
     {
      set_fam_text ("text to find");
      doc_search_f (cur_text_doc, gtk_entry_get_text (ent_search));
     }
}


void on_mni_kill_formatting_on_each_line (GtkMenuItem *menuitem,
                                          gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);

  if (buf)
     {
      gchar *temp = kill_formatting_on_each_line (buf, -1);
      doc_rep_sel (cur_text_doc, temp);
      g_free (buf);
      g_free (temp);
     }
}


void on_mni_file_crapbook (GtkMenuItem *menuitem,
                           gpointer user_data)
{
  if (! g_file_test (confile.crapbook_file, G_FILE_TEST_EXISTS))
     create_empty_file (confile.crapbook_file, _("you can put some crap here..."));

  handle_file (confile.crapbook_file, 0);
}


void on_mni_spellcheck (GtkMenuItem *menuitem,
                        gpointer user_data)
{
  if (! get_page_text()) return;
  do_hl_spell_check2 (cur_text_doc, gtk_widget_get_name (menuitem));
}


void on_mni_filter_kill_lesser (GtkMenuItem *menuitem,
                                gpointer user_data)
{
  if (! get_page_text()) return;
  
  set_fam_text ("2");
   
  gchar *buf = doc_get_sel (cur_text_doc);

  if (buf)
     {
      gchar *t = kill_lesser (buf, strtol (gtk_entry_get_text (ent_search), NULL, 10));
      doc_rep_sel (cur_text_doc, t);
      g_free (buf);
      g_free (t);
     }
}


void on_mni_filter_kill_more_than (GtkMenuItem *menuitem,
                                   gpointer user_data)
{
  if (! get_page_text()) return;
  
  set_fam_text ("2");
   
  gchar *buf = doc_get_sel (cur_text_doc);

  if (buf)
     {
      gchar *t = kill_larger (buf, strtol (gtk_entry_get_text (ent_search), NULL, 10));
      doc_rep_sel (cur_text_doc, t);
      g_free (buf);
      g_free (t);
     }
}


void on_mni_func_strings_sort_by_q (GtkMenuItem* menuitem,
                                    gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf) return;

  GList *temp = glist_from_string (buf);
  temp = glist_word_sort_mode (temp, 2);
  gchar *t = string_from_glist (temp);

  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
  glist_strings_free (temp);
}


void on_mni_tabs_menuitem_click (GtkMenuItem *menuitem,
                                gpointer user_data)
{
  t_note_page *doc = user_data;
  if (doc) 
     handle_file (doc->file_name, 0);
}

gchar* get_full_fname (const gchar *fname, const gchar *linkname)
{
  gchar *dir = g_path_get_dirname (fname);
  gchar *filename = create_full_path (linkname, dir);
  g_free (dir);
  return filename;
}


void on_mni_show_images_in_text (GtkMenuItem *menuitem,
                                 gpointer user_data)
{
  if (! get_page_text()) return;
  
  if (! g_file_test (cur_text_doc->file_name, G_FILE_TEST_EXISTS))
     return;

  gchar *f;
  gchar *t;
  GtkImage *image; 
  GdkPixbuf *p;

  GtkTextIter match_start;
  GtkTextIter match_end;

  gtk_text_buffer_get_start_iter (cur_text_doc->text_buffer, &match_start); 
  gtk_text_buffer_get_end_iter (cur_text_doc->text_buffer, &match_end); 

  while (gtk_text_iter_forward_search (&match_start, "src=\"", GTK_TEXT_SEARCH_TEXT_ONLY, &match_start, &match_end, NULL))
       {
        match_start = match_end;
        if (gtk_text_iter_forward_find_char (&match_end, find_quote, NULL, NULL))   
           {
            f = gtk_text_buffer_get_text (cur_text_doc->text_buffer, &match_start, &match_end, FALSE);
            
            t = get_full_fname (cur_text_doc->file_name, f);    
            if (t)
            if (is_image (t))
               {
                image = gtk_image_new_from_file (t);
                gtk_widget_show (image);
                gtk_text_buffer_insert_pixbuf (cur_text_doc->text_buffer,
                                               &match_end,
                                               gtk_image_get_pixbuf (image));

                gtk_widget_destroy (image); 
               }

            g_free (f);
            g_free (t);
           }
        match_start = match_end;
       }

  while (gtk_text_iter_forward_search (&match_start, "SRC=\"", GTK_TEXT_SEARCH_TEXT_ONLY, &match_start, &match_end, NULL))
       {
        match_start = match_end;
        if (gtk_text_iter_forward_find_char (&match_end, find_quote, NULL, NULL))   
           {
            f = gtk_text_buffer_get_text (cur_text_doc->text_buffer, &match_start, &match_end, FALSE);
            t = get_full_fname (cur_text_doc->file_name, f);    
            if (t)
            if (is_image (t))
               {
                image = gtk_image_new_from_file (t);
                gtk_widget_show (image);
                gtk_text_buffer_insert_pixbuf (cur_text_doc->text_buffer,
                                               &match_end,
                                               gtk_image_get_pixbuf (image));

                gtk_widget_destroy (image); 
               }

            g_free (f);
            g_free (t);  
           }
        match_start = match_end;
       }
}


void on_mni_nav_goto_recent_tab (GtkMenuItem* menuitem,
                                 gpointer user_data)
{
  gint t = gtk_notebook_get_current_page (notebook1);

  if (last_page != -1)
      gtk_notebook_set_current_page (notebook1, last_page);

  last_page = t;
}


void on_mni_nav_goto_selection (GtkMenuItem* menuitem,
                                gpointer user_data)
{
  last_page = gtk_notebook_get_current_page (notebook1);
  gtk_notebook_prev_page (notebook1);
}


void on_mni_nav_goto_prev_tab (GtkMenuItem* menuitem,
                               gpointer user_data)
{
  last_page = gtk_notebook_get_current_page (notebook1);
  gtk_notebook_prev_page (notebook1);
}


void on_mni_nav_goto_next_tab (GtkMenuItem* menuitem,
                               gpointer user_data)
{
  last_page = gtk_notebook_get_current_page (notebook1);
  gtk_notebook_next_page (notebook1);
}


void on_mni_view_hide_highlighting (GtkMenuItem* menuitem,
                                    gpointer user_data)
{
  if (! get_page_text()) return;
  remove_tags (cur_text_doc);
}


void on_mni_nav_goto_first_tab (GtkMenuItem* menuitem,
                                gpointer user_data)
{
  last_page = gtk_notebook_get_current_page (notebook1);
  gtk_notebook_set_current_page (notebook1, 0);
}


void on_mni_nav_goto_last_tab (GtkMenuItem* menuitem,
                               gpointer user_data)
{
  last_page = gtk_notebook_get_current_page (notebook1);
  gtk_notebook_set_current_page (notebook1, -1);
}


void on_mni_quest_find (GtkMenuItem* menuitem,
                        gpointer user_data)
{
  if (! get_page_text()) return;
  set_fam_text ("text to find");
  doc_search_f (cur_text_doc, gtk_entry_get_text (ent_search));
}


void on_mni_quest_find_next (GtkMenuItem* menuitem,
                             gpointer user_data)
{
  if (! get_page_text()) return;
  doc_search_f_next (cur_text_doc);
}


void on_mni_quest_find_ncase (GtkMenuItem* menuitem,
                              gpointer user_data)
{
  if (! get_page_text()) return;
  set_fam_text ("text to find");
  doc_search_f_ncase (cur_text_doc, gtk_entry_get_text (ent_search));
}


void on_mni_quest_find_next_ncase (GtkMenuItem* menuitem,
                                   gpointer user_data)
{
  if (! get_page_text()) return;
  doc_search_f_next_ncase (cur_text_doc);
}


void on_mni_user_menu_item (GtkMenuItem *menuitem,
                            gpointer user_data)
{
  gchar *t = g_hash_table_lookup (ht_user_menu, gtk_widget_get_name (menuitem));
  if (! t) 
     return;

  gchar *s = g_strdup (t);
  gchar *cmd = NULL;
  if ( get_page_text()) 
     {
      cmd = str_replace_all (s, "%s", cur_text_doc->file_name);       
      system (cmd);
      g_free (cmd);
     }
  g_free (s);
}


void on_mni_user_menu_open (GtkMenuItem *menuitem,
                            gpointer user_data)
{
  if (! g_file_test (confile.user_menu_file, G_FILE_TEST_EXISTS))
     create_empty_file (confile.user_menu_file, "kwrite=kwrite %s &");

  g_free (cur_settings.selected_enc);
  cur_settings.selected_enc = g_strdup ("UTF-8");
  open_file_std (confile.user_menu_file);
}

//n.p. Cypress Hill - Insane In The Brain
void on_mni_find_and_replace_wnd (GtkMenuItem *menuitem,
                                  gpointer user_data)
{
  if (! get_page_text()) return;

  if (sr_wnd && sr_wnd->window)
     gtk_window_present (sr_wnd);
  else
      sr_wnd = create_fr ();
}


void on_mni_rtfm (GtkMenuItem *menuitem,
                  gpointer user_data)
{
  run_doc_in_browser ();
}


void on_mni_crackerize (GtkMenuItem *menuitem,
                        gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *t = doc_get_sel (cur_text_doc);

  if (! t)
     return;

  gchar *x = str_crackerize (t);
 
  doc_rep_sel (cur_text_doc, x);
  g_free (t);
  g_free (x);
}


void on_mni_markup_stuff (GtkMenuItem *menuitem,
                          gpointer user_data) 
{
  if (! get_page_text()) return;
  
  gchar *n = gtk_widget_get_name (menuitem);

  gchar *t = NULL;
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf)
     {
      t = g_strconcat ("<", n, "></", n, ">", NULL);
      doc_insert_at_cursor (cur_text_doc, t);
      doc_move_cursor_backw_middle_tags (cur_text_doc);
     } 
  else
      {
       t = g_strconcat ("<", n, ">", buf, "</", n, ">", NULL);
       doc_rep_sel (cur_text_doc, t);
      }

  g_free (t);
  g_free (buf);
}


void on_mni_show_options (GtkMenuItem *menuitem,
                          gpointer user_data)
{
  wnd_options_create ();
}


void on_mni_new_kwas (GtkMenuItem *menuitem,
                      gpointer user_data)
{
   create_kwas_window ();
}


//n.p. Orbital - The Box
void on_mni_open_kwas_bookmarks_file (GtkMenuItem *menuitem,
                                      gpointer user_data)
{
//  g_free (cur_settings.selected_enc);
//  cur_settings.selected_enc = g_strdup ("UTF-8");
  cur_settings.selected_enc = ch_str (cur_settings.selected_enc, "UTF-8");
  open_file_std (confile.tea_kwas_bookmarks);
}


void on_mni_eol_to_crlf (GtkMenuItem* menuitem,
                          gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_buf (cur_text_doc->text_buffer);
  gchar *z = str_eol_to_crlf (buf);

  gtk_text_buffer_set_text (cur_text_doc->text_buffer, z, -1);

  g_free (z);
  g_free (buf); 
}


void on_mni_eol_to_lf (GtkMenuItem* menuitem,
                       gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_buf (cur_text_doc->text_buffer);
  gchar *z = str_replace_all (buf, "\r\n", "\n");
  
  gtk_text_buffer_set_text (cur_text_doc->text_buffer, z, -1);

  g_free (z);
  g_free (buf); 
  
}


void on_mni_templ_file_open_activate (GtkMenuItem *menuitem,
                                      gpointer user_data)
{
  GtkFileSelection *file_selector = gtk_file_selection_new (_("Open file:"));

  g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                    "clicked",
                    G_CALLBACK (file_open_callback_widget),
                    (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->cancel_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  gtk_file_selection_set_filename (file_selector, confile.templates_dir);
  gtk_window_resize (file_selector, get_value (confile.screen_w, confile.saveopen_w), get_value (confile.screen_h, confile.saveopen_h));
  gtk_widget_show (file_selector);
}


void on_mni_upcase_each_first_letter_ofw (GtkMenuItem *menuitem,
                                          gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *z = upcase_each_first_letter_ofw (cur_text_doc);
  if (z)
      doc_rep_sel (cur_text_doc, z);

  g_free (z);
}


void on_mni_nav_mplayer_jump (GtkMenuItem *menuitem,
                              gpointer user_data)
{
  if (! confile.cm_mplayer)
     return;    

  if (! get_page_text()) 
     return;

  if (! confile.current_movie)
     {
      log_to_memo (_("Before using that, go to File > Open different > Open movie."), NULL, LM_ERROR);
      return;
     }   

  GtkTextIter iter;
  GtkTextMark *m = gtk_text_buffer_get_insert (cur_text_doc->text_buffer);

  gtk_text_buffer_get_iter_at_mark (cur_text_doc->text_buffer, &iter, m);

  GtkTextIter end;
  GtkTextIter start;

  if
     (! gtk_text_iter_forward_search (&iter,
                                "\n\n",
                                GTK_TEXT_SEARCH_TEXT_ONLY,
                                &end,
                                NULL,
                                NULL))
     return; 



  if (! gtk_text_iter_backward_search (&end,
                                  "-->",
                                  GTK_TEXT_SEARCH_TEXT_ONLY,
                                  &iter,
                                  NULL,
                                  NULL))
      return;


  if (! gtk_text_iter_backward_search (&iter,
                                  "\n",
                                  GTK_TEXT_SEARCH_TEXT_ONLY,
                                  &start,
                                  NULL,
                                  NULL))
      return;

  gchar *t = gtk_text_iter_get_text (&start, &iter);
  if (! t)
     return;

  t = g_strstrip (t);

  gchar *s;
  gchar *m1;
  gchar *m2;

  gchar *cm = g_strdup (confile.cm_mplayer);

  if (strstr (cm, "@encoding"))
     {
      s = g_str_replace (cm, "@encoding", cur_text_doc->encoding);
      g_free (cm);
      cm = s;
     }

  if (strstr (cm, "@time"))
     {
      s = g_str_replace (cm, "@time", t);
      g_free (cm);
      cm = s;
     }

  if (strstr (cm, "@sub_file"))
     {
      s = g_str_replace (cm, "@sub_file", cur_text_doc->file_name);
      g_free (cm);
      cm = s;
     }

  if (strstr (cm, "@movie_file"))
     {
      s = g_str_replace (cm, "@movie_file", confile.current_movie);
      g_free (cm);
      cm = s;
     }
  
  //dbm (cm);
  system (cm);
//  g_spawn_command_line_async      (cm, NULL);

  g_free (t);
  g_free (cm);
}


static void file_open_movie_cb (GtkWidget *widget, gpointer user_data)
{
  gchar *fn = gtk_file_selection_get_filename (user_data);
 
  if (! fn)
     return;  

  g_free (confile.current_movie);
  confile.current_movie = g_strdup (fn);

}


void on_mni_open_open_movie (GtkMenuItem *menuitem,
                             gpointer user_data)
{
  GtkFileSelection *file_selector = gtk_file_selection_new (_("Open a movie:"));

  g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                    "clicked",
                    G_CALLBACK (file_open_movie_cb),
                    (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->cancel_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  gtk_window_resize (file_selector, get_value (confile.screen_w, confile.saveopen_w), get_value (confile.screen_h, confile.saveopen_h));

  gchar *t;
  gchar *dir;

  if (get_page_text())
  if (g_file_test (cur_text_doc->file_name, G_FILE_TEST_EXISTS))
     {
      t = g_path_get_dirname (cur_text_doc->file_name);
      dir = g_strconcat (t, G_DIR_SEPARATOR_S, NULL);
      gtk_file_selection_set_filename (file_selector, dir);
      g_free (dir);
      g_free (t);
     }

  gtk_widget_show (file_selector);
}


void on_mni_help_item (GtkMenuItem *menuitem,
                       gpointer user_data)
{
  //g_free (cur_settings.selected_enc);
  //cur_settings.selected_enc = g_strdup ("UTF-8");
  cur_settings.selected_enc = ch_str (cur_settings.selected_enc, "UTF-8");
  gchar *f = get_tea_doc_compose_name (gtk_widget_get_name (menuitem));
  open_file_std (f);
  g_free (f);
}

/*
static void file_open_base64_widget (GtkWidget *widget, gpointer user_data)
{
  gchar *fn = gtk_file_selection_get_filename (user_data);
 
  if (! fn)
     return;  

  run_decoder (fn, confile.tmp_file); 
  open_file_std (confile.tmp_file);
}


void on_mni_open_base64_activate (GtkMenuItem *menuitem,
                                  gpointer user_data)
{
  GtkFileSelection *file_selector = gtk_file_selection_new (_("Open a session:"));

  g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                    "clicked",
                    G_CALLBACK (file_open_base64_widget),
                    (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->ok_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_selector)->cancel_button),
                            "clicked",
                            G_CALLBACK (gtk_widget_destroy),
                            (gpointer) file_selector);

  GtkWidget *mni_enc = gtk_option_menu_new ();
  GtkWidget *mni_m = gtk_menu_new ();

  gtk_widget_show (mni_enc);
  gtk_option_menu_set_menu (mni_enc, mni_m);

  g_free (cur_settings.selected_enc);

  if (confile.do_enc_autodetect)
       cur_settings.selected_enc = g_strdup ("autodetect");
  else
      cur_settings.selected_enc = g_strdup ("UTF-8");

  GList *p = g_list_first (confile.iconv_encs);
  while (p)
        {
         mni_temp = new_menu_item ((gchar*) p->data, mni_m, on_mni_new_file_open_select_enc);
         p = g_list_next (p);
        }

  gtk_box_pack_start (file_selector->main_vbox, mni_enc, FALSE, FALSE, 0);
  gtk_option_menu_set_history (mni_enc, 0);


  gtk_window_resize (file_selector, get_value (confile.screen_w, confile.saveopen_w), get_value (confile.screen_h, confile.saveopen_h));
  //gtk_file_selection_set_filename (file_selector, confile.sessions_dir);

  gtk_widget_show (file_selector);
}
*/

void on_mni_insert_doctype (GtkMenuItem *menuitem,
                            gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *n = gtk_widget_get_name (menuitem);
  gchar *t;

  if (strcmp (n, "HTML 4.1 Transitional") == 0)
     t = g_strdup ("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/1999/REC-html401-19991224/loose.dtd\">");
  else 
  if (strcmp (n, "HTML 4.1 Strict") == 0)
     t = g_strdup ("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">");
  else
  if (strcmp (n, "HTML 4.1 Frameset") == 0)
     t = g_strdup ("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Frameset//EN\" \"http://www.w3.org/TR/1999/REC-html401-19991224/frameset.dtd\">");

  else
  if (strcmp (n, "XHTML 1.0 Strict") == 0)
     t = g_strdup ("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">");
  else
  if (strcmp (n, "XHTML 1.0 Transitional") == 0)
     t = g_strdup ("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">");
  else
  if (strcmp (n, "XHTML 1.0 Frameset") == 0)
     t = g_strdup ("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Frameset//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd\">");
  else
  if (strcmp (n, "WML 1.1") == 0)
     t = g_strdup ("<xml version='1.0'> \n<!DOCTYPE wml PUBLIC \"-//WAPFORUM//DTD WML 1.1//EN\" \"http://www.wapforum.org/DTD/wml_1.1.xml\">");
  
  doc_insert_at_cursor (cur_text_doc, t); 

  g_free (t);
}


void on_mni_entity (GtkMenuItem *menuitem,
                    gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *t = NULL;
  gchar *n = gtk_widget_get_name (menuitem);
  gchar *x = g_hash_table_lookup (ht_entities, n);
  if (x)
     t = g_strdup_printf ("&#%s;", x);

  doc_insert_at_cursor (cur_text_doc, t); 

  g_free (t);    
}


void on_mni_sessions_click (GtkMenuItem *menuitem,
                            gpointer user_data)
{
  gchar *f = g_strconcat (confile.sessions_dir, gtk_widget_get_name (menuitem), NULL);

  GList *list = load_file_to_glist (f);
  GList *t = g_list_first (list);

  while (t) 
       {
        execute_recent_item (t->data);
        t = g_list_next (t);
       }

  glist_strings_free (list);
  g_free (f);
}


void on_mni_templates_click (GtkMenuItem *menuitem,
                             gpointer user_data)
{
  gchar *f = g_strconcat (confile.templates_dir, gtk_widget_get_name (menuitem), NULL);
  gchar *t = str_file_read (f);

  doc_ins_to_new (t);

  g_free (f);
  g_free (t);
}


void on_mni_morse_encode (GtkMenuItem *menuitem,
                          gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *t = doc_get_sel (cur_text_doc);

  if (! t)
     return;

  gchar *x = morse_encode (t);
 
  doc_rep_sel (cur_text_doc, x);
  g_free (t);
  g_free (x);
}


void on_mni_morse_decode (GtkMenuItem *menuitem,
                          gpointer user_data)
{
  if (! get_page_text()) return;

  gchar *t = doc_get_sel (cur_text_doc);

  if (! t)
     return;

  gchar *x = morse_decode (t);
 
  doc_rep_sel (cur_text_doc, x);
  g_free (t);
  g_free (x);
}


void on_mni_set_hl_mode (GtkMenuItem *menuitem,
                         gpointer user_data)
{
  //n.p. Method Man - Tical - Biscuits
  if (! get_page_text()) return;
  cur_text_doc->hl_mode = ch_str (cur_text_doc->hl_mode, gtk_widget_get_name (menuitem));
  apply_hl (cur_text_doc);
}


typedef struct 
              {
               t_note_page *page;
               gchar *prefix;    
               GtkTextIter itstart;
               GtkTextIter itend;  
             } t_text_part;



static void on_mni_select_autocomp_item (GtkMenuItem *menuitem,
                                         gpointer user_data)
{
  if (! get_page_text()) return;
  t_text_part *p = user_data;

  gchar *t = gtk_widget_get_name (menuitem);

  gtk_text_buffer_delete (p->page->text_buffer,
                          &p->itstart,
                          &p->itend);

  gtk_text_buffer_insert (p->page->text_buffer,
                          &p->itstart,
                          t,
                          -1);
    
  g_free (p->prefix);
  g_free (p);
}


//code from gtktextview.c Copyright (C) 2000 Red Hat, Inc
//modified by roxton
static void set_autocomp_menu_pos (GtkMenu *menu,
                                   gint *x,
                                   gint *y,
                                   gboolean *push_in,
                                   gpointer user_data)
{
  t_text_part *p = user_data;

  GtkWidget *widget;
  GdkRectangle cursor_rect;
  GdkRectangle onscreen_rect;
  gint root_x, root_y;
  GtkTextIter iter;
  GtkRequisition req;      
  GdkScreen *screen;
  
  widget = GTK_WIDGET (p->page->text_view);
  screen = gtk_widget_get_screen (widget);

  gdk_window_get_origin (widget->window, &root_x, &root_y);

  gtk_text_view_get_iter_location (p->page->text_view,
                                   &p->itstart,
                                   //&iter,
                                   &cursor_rect);

  gtk_text_view_get_visible_rect (p->page->text_view, &onscreen_rect);
  
  gtk_widget_size_request (menu, &req);

  /* can't use rectangle_intersect since cursor rect can have 0 width */
  if (cursor_rect.x >= onscreen_rect.x &&
      cursor_rect.x < onscreen_rect.x + onscreen_rect.width &&
      cursor_rect.y >= onscreen_rect.y &&
      cursor_rect.y < onscreen_rect.y + onscreen_rect.height)
    {    
      gtk_text_view_buffer_to_window_coords (p->page->text_view,
                                             GTK_TEXT_WINDOW_WIDGET,
                                             cursor_rect.x, cursor_rect.y,
                                             &cursor_rect.x, &cursor_rect.y);

      *x = root_x + cursor_rect.x + cursor_rect.width;
      *y = root_y + cursor_rect.y + cursor_rect.height;
    }
  else
    {
      /* Just center the menu, since cursor is offscreen. */      
      *x = root_x + (widget->allocation.width / 2 - req.width / 2);
      *y = root_y + (widget->allocation.height / 2 - req.height / 2);      
    }

  /* Ensure sanity */
  *x = CLAMP (*x, root_x, (root_x + widget->allocation.width));
  *y = CLAMP (*y, root_y, (root_y + widget->allocation.height));

  *x = CLAMP (*x, 0, MAX (0, gdk_screen_get_width (screen) - req.width));
  *y = CLAMP (*y, 0, MAX (0, gdk_screen_get_height (screen) - req.height));
}
//ending of code from gtktextview.c


static void make_autocomp_menu (t_text_part *p)
{
   if (mni_autocomp)
      gtk_widget_destroy (mni_autocomp);   
   
   mni_autocomp = gtk_menu_new ();    

   GList *l = g_completion_complete (at_editor, p->prefix, NULL);
  
   if (l)
       {
        build_menu_wudata_from_glist (l, mni_autocomp, on_mni_select_autocomp_item, p);
        gtk_menu_popup (mni_autocomp, NULL, NULL, set_autocomp_menu_pos, p, 0, 0);
        gtk_widget_grab_focus (mni_temp);
        gtk_menu_item_select (mni_temp);
       }
}
//////////////////////

gboolean on_editor_keypress (GtkWidget *widget, GdkEventKey *event, gpointer data)
{
  t_note_page *page = data;
  gchar *t;
  gchar *s;
  GtkTextIter itstart;
  GtkTextIter itend;  
  

  t_text_part *p;

  guint32 k = gdk_keyval_to_unicode (event->keyval);

  if (confile.do_autorep && ht_autoreplace)

     {
      if ( g_unichar_ispunct (k) || g_unichar_isspace (k))
      t = doc_get_word_at_left (cur_text_doc, &itstart, &itend);
      if (! t)
         return FALSE;
         p = g_malloc (sizeof (t_text_part));
         p->page = page;
         p->itstart = itstart; 
         p->itend = itend; 
         p->prefix = g_strdup (t);

         s = g_hash_table_lookup (ht_autoreplace, t);  
         if (s)
         {  
         gtk_text_buffer_delete (p->page->text_buffer,
                          &p->itstart,
                          &p->itend);

          gtk_text_buffer_insert (p->page->text_buffer,
                          &p->itstart,
                          s,
                          -1);
         } 
        g_free (p->prefix);
         g_free (p);
 
         //make_autocomp_menu (p);
         g_free (t); 
         

         return FALSE;
      

     }

  /*if (confile.use_autocomp)
     {
      if (event->keyval == GDK_space) 
         {
          t = doc_get_word_at_left (cur_text_doc, &itstart, &itend);
          if (! t)
             return FALSE;

          gl_autocomp = glist_add_not_dup (gl_autocomp, t, confile.autocomp_wordlen_min);

          if (gl_autocomp)
             {
              g_completion_clear_items (at_editor);
              g_completion_add_items (at_editor, gl_autocomp);
             }

          g_free (t); 
          return FALSE;
         } 

     if (event->keyval == GDK_Alt_R) 
        {
         t = doc_get_word_at_left (cur_text_doc, &itstart, &itend);
         if (! t)
            return FALSE;
         p = g_malloc (sizeof (t_text_part));
         p->page = page;
         p->itstart = itstart; 
         p->itend = itend; 
         p->prefix = g_strdup (t);
         make_autocomp_menu (p);
         g_free (t); 
         return FALSE;
        } 
    }
*/

////////////////////  

  if (event->keyval == GDK_Return)
     if (confile.use_auto_indent) 
        {
         indent_real(page->text_view);
         return TRUE;
        }

  //if (event->keyval == GDK_ISO_Left_Tab)
  if (event->keyval == GDK_Tab)
     {
      doc_indent_selection (page, FALSE);
      return TRUE; 
     }


 return FALSE;
}


void on_mni_save_pos (GtkMenuItem *menuitem,
                      gpointer user_data)
{
  if (! get_page_text()) return;
  cur_text_doc->position = editor_get_pos (cur_text_doc);
}


void on_mni_jump_to_saved_pos (GtkMenuItem *menuitem,
                               gpointer user_data)
{
  if (! get_page_text()) return;
  editor_set_pos (cur_text_doc, cur_text_doc->position);
}


void on_mni_unindent (GtkMenuItem *menuitem,
                      gpointer user_data)
{
  if (! get_page_text()) return;
  doc_indent_selection (cur_text_doc, TRUE);
}


void on_mni_indent (GtkMenuItem *menuitem,
                    gpointer user_data)
{
  if (! get_page_text()) return;
  doc_indent_selection (cur_text_doc, FALSE);
}


void on_mni_autorep_file_open (GtkMenuItem *menuitem,
                               gpointer user_data)
{
  cur_settings.selected_enc = ch_str (cur_settings.selected_enc, "UTF-8");
  open_file_std (confile.tea_autoreplace);
}


void on_mni_strings_remove_leading_whitespaces (GtkMenuItem *menuitem,
                                                gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf) return;

  GList *temp = glist_from_string (buf);
  GList *temp2 = each_line_remove_lt_spaces (temp, 0);
  gchar *t = string_from_glist (temp2);

  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
  glist_strings_free (temp);
  glist_strings_free (temp2);
}


void on_mni_strings_remove_trailing_whitespaces (GtkMenuItem *menuitem,
                                                 gpointer user_data)
{
  if (! get_page_text()) return;
  
  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf) return;

  GList *temp = glist_from_string (buf);
  GList *temp2 = each_line_remove_lt_spaces (temp, 0);
  gchar *t = string_from_glist (temp2);

  doc_rep_sel (cur_text_doc, t);

  g_free (buf);
  g_free (t);
  glist_strings_free (temp);
  glist_strings_free (temp2);
}


void on_mni_edit_append_to_crapbook (GtkMenuItem *menuitem,
                                     gpointer user_data)
{
  if (! get_page_text()) return;
  
  t_note_page *p;
  GtkTextIter iter;

  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf) return;

  gchar *t = g_strconcat (buf, "\n", NULL);  

  gint i = get_n_page_by_filename (confile.crapbook_file);

  if (i != -1)
     {
      p = get_page_by_index (i);
      gtk_text_buffer_get_iter_at_offset (p->text_buffer, &iter, 0);
      gtk_text_buffer_insert (p->text_buffer, &iter, t, -1);
      g_free (buf);
      g_free (t);
      return;
     } 

  GList *l = load_file_to_glist (confile.crapbook_file);

  l = g_list_prepend (l, t);

  glist_save_to_file (l, confile.crapbook_file);

  glist_strings_free (l);
}


void on_mni_test (GtkMenuItem *menuitem,
                  gpointer user_data)
{
  if (! get_page_text()) return;
  
  t_note_page *p;
  GtkTextIter iter;

  gchar *buf = doc_get_sel (cur_text_doc);

  if (! buf) return;

  gchar *t = g_strconcat (buf, "\n", NULL);  

  gint i = get_n_page_by_filename (confile.crapbook_file);

  if (i != -1)
     {
      p = get_page_by_index (i);
      gtk_text_buffer_get_iter_at_offset (p->text_buffer, &iter, 0);
      gtk_text_buffer_insert (p->text_buffer, &iter, t, -1);
      g_free (buf);
      g_free (t);
      return;
     } 

  GList *l = load_file_to_glist (confile.crapbook_file);

  l = g_list_prepend (l, t);

  glist_save_to_file (l, confile.crapbook_file);
  glist_strings_free (l);
}


void on_mni_load_last_file (GtkMenuItem *menuitem,
                            gpointer user_data)
{
  if (! recent_list)
    return;

  execute_recent_item (recent_list->data); 
}

